/*
 * @(#)AbstractTransferAdapter.java	1.0 2001/03/12
 *
 * Copyright (C) 2001 by Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter;


// Java Imports
import java.io.InputStream;
import java.io.OutputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.JDialog;

import net.sourceforge.journeyftp.adapter.resources.AdapterResourceBundle;


/**
 * Base functionality for adapters.
 *
 * @author  Maru Newby
 * @version 1.0, 2001/03/29
 */
abstract public class AbstractTransferAdapter implements TransferAdapter,
                                                         FileTransferObserver
{
    // Class variables
    // Error keys
    public final static String ERR_ADAPTER_CONNECTION_FAILED =
                                               "Adapter.ConnectionFailed";
    public final static String ERR_ADAPTER_OPERATION_FAILED =
                                               "Adapter.OperationFailed";
    public final static String ERR_ADAPTER_OPERATION_NOT_SUPPORTED =
                                               "Adapter.OperationNotSupported";

    public final static String ERR_CONNECT_CONNECTION_FAILED =
                                               "Connect.ConnectionFailed";
    public final static String ERR_CONNECT_CONNECTION_REFUSED =
                                               "Connect.ConnectionRefused";
    public final static String ERR_CONNECT_HOST_NOT_FOUND =
                                               "Connect.HostNotFound";
    public final static String ERR_CONNECT_PERMISSION_DENIED =
                                               "Connect.PermissionDenied";

    public final static String ERR_CWD_RESOURCE_NOT_FOUND =
                                       "SetWorkingDirectory.ResourceNotFound";
    public final static String ERR_CWD_INCORRECT_FILE_TYPE =
                                       "SetWorkingDirectory.IncorrectFileType";
    public final static String ERR_CWD_PERMISSION_DENIED =
                                       "SetWorkingDirectory.PermissionDenied";

    public final static String ERR_GET_FILE_RESOURCE_NOT_FOUND =
                                       "GetFile.ResourceNotFound";
    public final static String ERR_GET_FILE_INCORRECT_OFFSET =
                                       "GetFile.IncorrectOffset";
    public final static String ERR_GET_FILE_INCORRECT_FILE_TYPE =
                                       "GetFile.IncorrectFileType";
    public final static String ERR_GET_FILE_OPERATION_FAILED =
                                       "GetFile.OperationFailed";
    public final static String ERR_GET_FILE_PERMISSION_DENIED =
                                       "GetFile.PermissionDenied";
    public final static String ERR_GET_FILE_RESUME_NOT_SUPPORTED =
                                       "GetFile.ResumeNotSupported";

    public final static String ERR_LIST_RESOURCE_NOT_FOUND =
                                               "List.ResourceNotFound";
    public final static String ERR_LIST_PARSE_ERROR =
                                               "List.ParseError";

    public final static String ERR_LIST_FILE_RESOURCE_NOT_FOUND =
                                               "ListFile.ResourceNotFound";
    public final static String ERR_LIST_FILE_INCORRECT_FILE_TYPE =
                                               "ListFile.IncorrectFileType";

    public final static String ERR_MKDIR_FILE_EXISTS = "CreateDir.FileExists";
    public final static String ERR_MKDIR_DIR_EXISTS = "CreateDir.DirExists";
    public final static String ERR_MKDIR_PERMISSION_DENIED =
                                                "CreateDir.PermissionDenied";

    public final static String ERR_PARSE_LIST =
                                               "ParseList.ParseError";

    public final static String ERR_PUT_FILE_DIR_EXISTS =
                                       "PutFile.DirectoryExists";
    public final static String ERR_PUT_FILE_FILE_EXISTS =
                                       "PutFile.FileExists";
    public final static String ERR_PUT_FILE_INCORRECT_OFFSET =
                                       "PutFile.IncorrectOffset";
    public final static String ERR_PUT_FILE_PERMISSION_DENIED =
                                       "PutFile.PermissionDenied";
    public final static String ERR_PUT_FILE_RESUME_NOT_SUPPORTED =
                                       "PutFile.ResumeNotSupported";

    public final static String ERR_REMOVE_DIR_NOT_EMPTY =
                                           "Remove.DirectoryNotEmpty";
    public final static String ERR_REMOVE_INCORRECT_FILE_TYPE =
                                           "Remove.IncorrectFileType";
    public final static String ERR_REMOVE_PERMISSION_DENIED =
                                           "Remove.PermissionDenied";
    public final static String ERR_REMOVE_RESOURCE_NOT_FOUND =
                                           "Remove.ResourceNotFound";

    public final static String ERR_RENAME_DIR_EXISTS =
                                       "Rename.DirectoryExists";
    public final static String ERR_RENAME_FILE_EXISTS =
                                       "Rename.FileExists";
    public final static String ERR_RENAME_PERMISSION_DENIED =
                                       "Rename.PermissionDenied";
    public final static String ERR_RENAME_RESOURCE_NOT_FOUND =
                                       "Rename.ResourceNotFound";

    public final static String ERR_TRANSFER_ABORTED =
                                       "Transfer.TransferAborted";
    public final static String ERR_TRANSFER_ABORT_FAILURE =
                                       "Transfer.TransferAbortFailure";


    // Operation failure messages
    public final static String OP_CONNECT = "connecting";
    public final static String OP_CWD = "setting working directory";
    public final static String OP_GET_FILE = "receiving a file";
    public final static String OP_LIST = "listing remote files";
    public final static String OP_MKDIR = "creating a directory";
    public final static String OP_PUT_FILE = "sending a file";
    public final static String OP_PWD =
                                       "getting the current working directory";
    public final static String OP_REMOVE = "removing a file/directory";
    public final static String OP_RENAME = "renaming a file";
    public final static String OP_XFER_SETUP = "setting up a file transfer";

    // Shell-style protocol messages (telnet, ftp)
    public final static String SH_DIR_NOT_EMPTY = "irectory not empty";
    public final static String SH_FILE_EXISTS = "ile exists";
    public final static String SH_IS_A_DIRECTORY = "is a directory";
    public final static String SH_LOGIN_INCORRECT = "ogin incorrect";
    public final static String SH_NO_SUCH_FILE = "o such file or directory";
    public final static String SH_NOT_A_DIRECTORY = "ot a directory";
    public final static String SH_PERMISSION_DENIED = "ermission denied";
    public final static String SH_COMMAND_NOT_FOUND = "ommand not found";


    // Instance variables
    protected Vector       _observers = new Vector();
    protected FileTransfer _transfer  = null;


    /**
     * Default no-arg constructor.
     */
    protected AbstractTransferAdapter()
    {
        addObserver(this);
    }

    /**
     * Aborts an in progress file transfer.
     */
    public void abortTransfer()
    {
        if (isTransferInProgress())
            _transfer.abortTransfer();

    } // abortTransfer()


    /**
     * Gives external entities the ability to be notified when file events
     * occur.
     *
     * @param observer      the observer to add
     */
    public void addObserver(FileTransferObserver observer)
    {
        _observers.add(observer);

    } // end: addTransferObserver


    /**
     * Checks that a file or directory with the same name as the one specified
     * does not exist.
     *
     * @param dirName       the name of the directory
     * @exception DirectoryExistsException      a directory with the same name
     *                                          already exists
     * @exception FileExistsException           a file with the same name
     *                                          already exists
     */
    public void checkDirectoryExists(String dirName)
                                     throws DirectoryExistsException,
                                            FileExistsException
    {
        // Check if a file/directory already exist
        VirtualFile file = null;
        boolean isDir = false;
        try
        {
            file = listFile(dirName);
        }
        catch (IncorrectFileTypeException ifte)
        {
            isDir = true;
        }
        catch (Exception ex)
        {
            // Ignore
        }

        // If a directory already exists
        if (isDir)
           throw (new DirectoryExistsException(
                                   getString(ERR_MKDIR_DIR_EXISTS, dirName)));

        // If a file already exists
        if (file != null)
            throw (new FileExistsException(
                                   getString(ERR_MKDIR_FILE_EXISTS, dirName)));

    } // end: checkDirectoryExists


    /**
     * This method is called every time the <code>FileTransfer</code> object
     * transfers a block of bytes, allowing observers to monitor the status of
     * a file transfer.
     *
     * @param bytes         the number of bytes transferred
     */
    public void dataTransferred(long bytes)
    {
        // Do nothing
    }


    /**
     * Returns a prepared data connection for file transfers.
     *
     * @return              the data connection
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    abstract protected FileTransferConnection getConnection()
                                              throws ConnectionFailedException,
                                                     OperationFailedException;


    /**
     * Retrieves the identified filename from the current working directory of
     * the server and writes it to the given output stream.
     *
     * If the offset is non-zero, the contents of that file will be written to
     * the output stream from that offset.  It is assumed that the output stream
     * has already been set to the desired offset.
     *
     * Note that this method is designed to be asynchronous - using threads
     * and the <code>FileTransfer</code> object it should initiate a transfer
     * and communicate via the <code>TransferObserver</code> interface.
     *
     * @param file          the <code>VirtualFile</code> pointing to the file
     *                      to retrieve
     * @param out           the output stream to write the file to
     * @param offset        the offset, in bytes, to read the file from
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    public void getFile(VirtualFile file, OutputStream out, long offset)
                        throws ConnectionFailedException,
                               OperationFailedException
    {
        // TransferAdapterException is thrown from this method
        FileTransferConnection connection = getConnection();

        // Create and initialize the file receiver
        FileReceiver receiver = getFileReceiver();
        receiver.init(file.getName(), out, offset, connection);
        receiver.setVirtualFile(file);
        Enumeration enum = _observers.elements();
        while (enum.hasMoreElements())
            receiver.addObserver((FileTransferObserver)enum.nextElement());

        // Initialize the reference to the transfer
        _transfer = receiver;

        // Initialize and run the asynchronous transfer object
        Thread thread = new Thread(receiver);
        thread.run();

    } // end: getFile


    /**
     * Returns an uninitialied <code>FileReceiver</code> object to transfer
     * a file from the server.
     *
     * @return              the file receiver
     * @exception OperationFailedException  the adapter is not able to provide
     *                                      the receiver
     */
    abstract protected FileReceiver getFileReceiver()
                                               throws OperationFailedException;


    /**
     * Returns an uninitialied <code>FileSender</code> object to transfer
     * a file to the server.
     *
     * @return              the file sender
     * @exception OperationFailedException  the adapter is not able to provide
     *                                      the sender
     */
    abstract protected FileSender getFileSender()
                                               throws OperationFailedException;


    /**
     * Retrieves a resource string from the default bundle.
     *
     * @param key           the key of the resource string
     * @return              the resource string
     */
    protected String getString(String key)
    {
        return (AdapterResourceBundle.getResourceString(key));

    } // end: getString


    /**
     * Retrieves a resource string from the default bundle and substitutes the
     * given parameter.
     *
     * @param key           the key of the resource string
     * @param param1        a substitution parameter
     * @return              the resource string
     */
    protected String getString(String key, String param1)
    {
        return (AdapterResourceBundle.getResourceString(key,
                                                     new String[] { param1 }));

    } // end: getString


    /**
     * Retrieves a resource string from the default bundle and substitutes the
     * given parameters.
     *
     * @param key           the key of the resource string
     * @param param1        the first substitution parameter
     * @param param2        the second substitution parameter
     * @return              the resource string
     */
    protected String getString(String key, String param1, String param2)
    {
        return (AdapterResourceBundle.getResourceString(key,
                                                     new String[] { param1,
                                                                    param2 }));

    } // end: getString


    /**
     * Retrieves a resource string from the default bundle and substitutes the
     * given parameters.
     *
     * @param key           the key of the resource string
     * @param param1        the first substitution parameter
     * @param param2        the second substitution parameter
     * @param param3        the third substitution parameter
     * @return              the resource string
     */
    protected String getString(String key,
                            String param1,
                            String param2,
                            String param3)
    {
        return (AdapterResourceBundle.getResourceString(key,
                                                     new String[] { param1,
                                                                    param2,
                                                                    param3 }));

    } // end: getString


    /**
     * Indicates whether the adapter is currently transfering a file or not.
     *
     * @return              whether the adapter is currently transferring a file
     */
    public boolean isTransferInProgress()
    {
        return (_transfer != null);

    } // end: isTransferInProgress


    /**
     * Returns an array of <code>VirtualFile</code> objects, representing the
     * files and directories contained in the current working directory of the
     * server.
     *
     * @param fileName      the name of the file to list.  If not provided, the
     *                      current working directory will be listed
     * @return              the array of <code>VirtualFile</code> objects
     * @exception ConnectionFailedException     an connection-related error
     *                                          prevented successful completion
     *                                          of the operation
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception ResourceNotFoundException     the working directory does not
     *                                          contain any files or the list
     *                                          cannot be parsed
     */
    protected VirtualFile[] list(String fileName)
                                 throws ConnectionFailedException,
                                        OperationFailedException,
                                        ResourceNotFoundException
    {
        System.out.println("List(String) is not implemented");
        return (null);

    } // end: list


    /**
     * Returns an array of <code>VirtualFile</code> objects, representing the
     * files and directories contained in the current working directory of the
     * server.
     *
     * @return              the array of <code>VirtualFile</code> objects
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception ResourceNotFoundException     the working directory does not
     *                                          contain any files or the list
     *                                          cannot be parsed
     */
    public VirtualFile[] list() throws ConnectionFailedException,
                                       OperationFailedException,
                                       ResourceNotFoundException
    {
        return (list(""));

    } // end: list


    /**
     * Returns a <code>VirtualFile</code> object, representing the specified
     * file.
     *
     * @return              a <code>VirtualFile</code> object of the desired
     *                      file
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception IncorrectFileTypeException    a directory was specified for
     *                                          listing
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception ResourceNotFoundException     the identified file does not
     *                                          exist
     */
    public VirtualFile listFile(String fileName)
                                throws ConnectionFailedException,
                                       IncorrectFileTypeException,
                                       OperationFailedException,
                                       ResourceNotFoundException
    {
        VirtualFile[] dirList = null;
        // Retrieve the listing
        try
        {
            dirList = list(fileName);
        }
        catch (ResourceNotFoundException rnfe)
        {
            throw (new ResourceNotFoundException(
                                 getString(ERR_LIST_RESOURCE_NOT_FOUND,
                                           fileName)));
        }

        // Check if a directory was listed
        boolean dirListed = false;

        // If more than one file is returned, the contents of a directory
        // are being listed
        if (dirList.length > 1)
            dirListed = true;

        // If the first entry is a directory
        VirtualFile file = dirList[0];
        if (file.isDirectory())
            dirListed = true;

        // Throw an exception if a directory was listed
        if (dirListed)
            throw (new IncorrectFileTypeException(
                                getString(ERR_LIST_FILE_INCORRECT_FILE_TYPE,
                                          fileName)));

        return (file);

    } // end: listFile


    /**
     * Parses a unix-style directory listing of the format:
     *
     * permissions, hardlinks, owner, group, size, date modified, filename
     * drwxrwxrwx 4 root root    5444 Apr 3 12:11 filename
     *
     * A <code>VirtualFile</code> object is created for each entry.
     *
     * Note:
     *      - only valid directory listings are parsed.
     *      - the '.' and '..' directories are ignored
     *
     * @param listing       the directory listing to parse
     * @return              an array of <code>VirtualFile</code> objects
     *                      representing the contents of the directory listing
     * @exception ListFormatException   if unable to parsing any of the
     *                                  directory list items
     */
    public VirtualFile[] parseList(String listing)
                                   throws ListFormatException
    {
        return (parseList(listing, false));

    } // end: parseList


    /**
     * Parses a unix-style directory listing of the format:
     *
     * permissions, hardlinks, owner, group, size, date modified, filename
     * drwxrwxrwx 4 root root    5444 Apr 3 12:11 filename
     *
     * A <code>VirtualFile</code> object is created for each entry.
     *
     * Note:
     *      - only valid directory listings are parsed.
     *      - the '.' and '..' directories are ignored
     *
     * @param listing       the directory listing to parse
     * @param telnetCodes   removes telnet formatting codes from the names
     *                      of the entries
     * @return              an array of <code>VirtualFile</code> objects
     *                      representing the contents of the directory listing
     * @exception ListFormatException   if unable to parsing any of the
     *                                  directory list items
     */
    public VirtualFile[] parseList(String listing,
                                   boolean telnetCodes )
                                   throws ListFormatException
    {
        final String OLD_FILE_DATE = "MMM dd yyyy";
        final String NEW_FILE_DATE = "MMM dd hh:mm yyyy";
        final int    CURRENT_YEAR  = Calendar.getInstance().get(Calendar.YEAR);

        Vector dirContents = new Vector();
        StringTokenizer lines = new StringTokenizer(listing, "\n");
        StringTokenizer fileInfo = null;
        boolean failed = true;

        boolean formatDetermined = false;
        byte[] offsets = new byte[7];

        // Process each line in the output
        while (lines.hasMoreTokens())
        {
            VirtualFile temp = new VirtualFile();
            String listLine = lines.nextToken().trim();

            // If the line is a valid file listing
            if (listLine != null && (listLine.startsWith("d") ||
                                     listLine.startsWith("-")))
            {
                fileInfo = new StringTokenizer(listLine);

                // If the number of tokens is few than expected, error
                if (fileInfo.countTokens() < 8)
                    throw (new ListFormatException(
                                    getString(ERR_PARSE_LIST,
                                             "not enough tokens being present",
                                             listLine)));

                // Directory
                temp.setIsDirectory(listLine.startsWith("d"));

                // Ignore the following properties
                // permissions       // # hard links      // owner
                fileInfo.nextToken();fileInfo.nextToken();fileInfo.nextToken();
                // group
                fileInfo.nextToken();

                // size
                try
                {
                    temp.setSize(Long.parseLong(fileInfo.nextToken()));
                }
                catch (Exception ex)
                {
                    throw (new ListFormatException(
                                    getString(ERR_PARSE_LIST,
                                              "an inability to parse the size",
                                              listLine)));
                }

                // Month + day
                String monthDay = fileInfo.nextToken() + " " +
                                  fileInfo.nextToken();

                // Time/Year
                String timeYear = fileInfo.nextToken();

                String dateFormatString;
                // Determine the format to parse the date
                // Previous years, have year instead of time
                if (timeYear.indexOf(":") == -1)
                    dateFormatString = OLD_FILE_DATE;
                // Current year, have to add the year
                else
                {
                    dateFormatString = NEW_FILE_DATE;
                    timeYear = timeYear + " " + CURRENT_YEAR;
                }

                // Create the date string and its formatter
                String dateModified = monthDay + " " + timeYear;
                DateFormat dateFormat = new SimpleDateFormat(dateFormatString);

                // Try to format the date
                try
                {
                    temp.setDateModified(dateFormat.parse(dateModified));
                }
                catch (Exception ex)
                {
                    throw (new ListFormatException(
                                    getString(ERR_PARSE_LIST,
                                             "an inability to parse the date",
                                             listLine)));
                }

                // Name
                String name = fileInfo.nextToken().trim();

                /**
                 * Telnet formatting codes consist of [<num>m<filename>[0m
                 * prefixing the file name, and <cr>[0m] suffixing the name.
                 * They need to be removed.
                 */
                if (telnetCodes)
                {
                    int prefixEnd = name.indexOf("m");
                    int suffixStart = name.lastIndexOf("[");
                    name = name.substring(prefixEnd + 1, suffixStart - 1);
                }

                if (!name.equals(".") && !name.equals(".."))
                {
                    temp.setName(name);
                    // Add the virtual file to the directory
                    dirContents.add( temp );
                }

                // Clear the tokenizer
                fileInfo = null;
            }
        }

        VirtualFile[] dirList = new VirtualFile[dirContents.size()];
        dirContents.copyInto(dirList);

        return (dirList);

    } // end: parseList


    /**
     * Sends the data from the provided input stream to the server, where it
     * will be written to a file identified by fileName in the current working
     * directory of the server.  If the file already exists on the server, it
     * will be overwritten if permitted.
     *
     * If the offset is non-zero, the contents of that file will be written to
     * the server from that offset.  It is assumed that the input stream
     * has already been set to the desired offset.
     *
     * Note that this method is designed to be asynchronous - using threads
     * and the <code>FileTransfer</code> object it should initiate a transfer
     * and communicate via the <code>TransferObserver</code> interface.
     *
     * @param fileName      the name of the file to be sent to the server
     * @param in            the input stream to read the file from
     * @param offset        the offset, in bytes, to write the file from
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    public void putFile(String fileName, InputStream in, long offset)
                        throws ConnectionFailedException,
                               OperationFailedException
    {
        FileTransferConnection connection = getConnection();

        // Create and initialize the file receiver
        FileSender sender = getFileSender();
        sender.init(fileName, in, offset, connection);
        Enumeration enum = _observers.elements();
        while (enum.hasMoreElements())
            sender.addObserver((FileTransferObserver)enum.nextElement());

        _transfer = sender;

        // Initialize and run the asynchronous transfer object
        Thread thread = new Thread(sender);
        thread.run();

    } // end: putFile


    /**
     * Removes an observer.
     *
     * @param observer      the observer to remove
     */
    public void removeObserver(FileTransferObserver observer)
    {
        _observers.remove(observer);

    } // end: removeObserver


    /**
     * Helper method to throw an exception if an operation fails.
     *
     * @param operation     the operation being attempted
     * @exception ConnectionFailedException     if the user has been
     *                                          disconnected
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    public void operationFailed(String operation)
                                              throws ConnectionFailedException,
                                                     OperationFailedException
    {
        operationFailed(operation, null, "");

    } // end: operationFailed


    /**
     * Helper method to throw an exception if an operation fails.
     *
     * @param operation     the operation being attempted
     * @parma msg           the detailed reason the operation failed
     * @exception ConnectionFailedException     if the user has been
     *                                          disconnected
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    public void operationFailed(String operation, String msg)
                                              throws ConnectionFailedException,
                                                     OperationFailedException
    {
        operationFailed(operation, null, msg);

    } // end: operationFailed


    /**
     * Helper method to throw an exception if an operation fails.
     *
     * @param operation     the operation being attempted
     * @param ex            the exception that failed the operation
     * @exception ConnectionFailedException     if the user has been
     *                                          disconnected
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    public void operationFailed(String operation, Exception ex)
                                              throws ConnectionFailedException,
                                                     OperationFailedException
    {
        operationFailed(operation, ex, "");

    } // end: operationFailed


    /**
     * Helper method to throw the connection failed message.
     *
     * @param operation     the operation being attempted
     * @param ex            the exception that failed the operation
     * @parma msg           the error message describing the operation failure
     * @exception ConnectionFailedException     throws an error if the user
     *                                          has been disconnected
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    public void operationFailed(String operation, Exception ex, String msg)
                                              throws ConnectionFailedException,
                                                     OperationFailedException
    {
        if (isConnected())
            throw (new OperationFailedException(
                         getString(ERR_ADAPTER_OPERATION_FAILED, operation),
                         ex,
                         msg));
        else
            throw (new ConnectionFailedException(
                         getString(ERR_ADAPTER_CONNECTION_FAILED, operation),
                         ex,
                         msg));

    } // end: operationFailed


    /**
     * Indicates that a file transfer has been completed successfully.
     */
    public void transferSuccessful()
    {
        _transfer = null;

    } // end: transferSuccessful


    /**
     * Indicates that a file transfer has terminated.  The exception can be
     * used to determine why the transfer terminated.
     *
     * @param ex        the exception indicating the reason for the termination
     */
    public void transferTerminated(TransferAdapterException ex)
    {
        _transfer = null;

    } // end: transferTerminated


} // end: AbstractTransferAdapter
