/*
 * @(#)FileReceiver.java	1.0 2001/03/12
 *
 * Copyright (C) 2001 by Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter;


// Java imports
import java.io.OutputStream;


/**
 * Receives a file asynchronously.  This class minimizes the use of constructors
 * for initialization (use init() instead) to make subclassing a little easier.
 *
 * Subclasses only have to implement the getFile method.
 *
 * @author  Maru Newby
 * @version 1.0, 2001/03/13
 */
abstract public class FileReceiver extends FileTransfer
{
    // Instance variables
    private OutputStream _out = null;


    /**
     * Accessor for the output stream.
     *
     * @return              the output stream
     *
     */
    public OutputStream getOutputStream()
    {
        return (_out);

    } // end: getOutputStream


    /**
     * Initializes the transfer object with all the values necessary to
     * perform a file transfer, minus the connection.
     *
     * @param fileName      the name of the file to transfer
     * @param out           the output stream to write to
     * @param offset        the offset to recieve the file from
     */
    public void init(String fileName, OutputStream out, long offset)
    {
        setFileName(fileName);
        setOutputStream(out);
        setOffset(offset);

    } // end: init


    /**
     * Initializes the transfer object with all the values necessary to
     * perform a file transfer.
     *
     * @param fileName      the name of the file to transfer
     * @param out           the output stream to write to
     * @param offset        the offset to recieve the file from
     * @param connection    the transfer connection that will be used
     *                      to transfer the file
     */
    public void init(String fileName, OutputStream out, long offset,
                     FileTransferConnection connection)
    {
        init(fileName, out, offset);
        this.setConnection(connection);

    } // end: init


    /**
     * Retrieves the identified filename from the current working directory of
     * the server and writes it to an output stream.
     *
     * If the offset is non-zero, the contents of that file will be written to
     * the output stream from that offset.  It is assumed that the output stream
     * has already been set to the desired offset.
     *
     * @exception TransferAdapterException  an error occured that terminated
     *                                      the file transfer
     */
    abstract public void getFile() throws TransferAdapterException;


    /**
     * Notifies observers when the transfer is completed successfully.
     */
    public void notifyTransferSuccessful()
    {
        try
        {
            _out.flush();
            _out.close();
        }
        catch (Exception exe) {}
        super.notifyTransferSuccessful();

    } // end: notifyTransferSuccessful


    /**
     * Notifies observers when the transfer is terminated.
     *
     * @param ex            the exception that terminated the transfer
     */
    public void notifyTransferTerminated(TransferAdapterException ex)
    {
        try
        {
            _out.flush();
            _out.close();
        }
        catch (Exception exe) {}
        _out = null;
        super.notifyTransferTerminated(ex);

    } // end: notifyTransferTerminated


    /**
     * This method is run when the thread holding it is started.
     */
    public void run()
    {
        try
        {
            getFile();
            this.notifyTransferSuccessful();
        }
        catch (TransferAdapterException tae)
        {
            this.notifyTransferTerminated(tae);
        }

    } // end: run


    /**
     * Mutator for the output stream.
     *
     * @param out            the output stream
     *
     */
    public void setOutputStream(OutputStream out)
    {
        _out = out;

    } // end: setOutputStream


} // end: FileReciever