/*
 * @(#)FileSender.java	1.0 2001/03/12
 *
 * Copyright (C) 2001 by Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter;


// Java imports
import java.io.InputStream;



/**
 * Sends a file asynchronously.  This class minimizes the use of constructors
 * for initialization (use init() instead) to make subclassing a little easier.
 *
 * Subclasses only have to implement the putFile method.
 *
 * @author  Maru Newby
 * @version 1.0, 2001/03/13
 */
abstract public class FileSender extends FileTransfer
{
    // Instance variables
    private InputStream _in = null;


    /**
     * Accessor for the input stream.
     *
     * @return              the input stream
     *
     */
    public InputStream getInputStream()
    {
        return (_in);

    } // end: getInputStream


    /**
     * Initializes the transfer object with all the values necessary to
     * perform a file transfer, minus the connection.
     *
     * @param fileName      the name of the file to transfer
     * @param in            the input stream to read from
     * @param offset        the offset to send the file from
     */
    public void init(String fileName, InputStream in, long offset)
    {
        setFileName(fileName);
        setInputStream(in);
        setOffset(offset);

    } // end: init


    /**
     * Initializes the transfer object with all the values necessary to
     * perform a file transfer.
     *
     * @param fileName      the name of the file to transfer
     * @param in            the input stream to read from
     * @param offset        the offset to send the file from
     * @param connection    the transfer connection that will be used
     *                      to transfer the file
     */
    public void init(String fileName, InputStream in, long offset,
                     FileTransferConnection connection)
    {
        init(fileName, in, offset);
        this.setConnection(connection);

    } // end: init


    /**
     * Notifies observers when the transfer is completed successfully.
     */
    public void notifyTransferSuccessful()
    {
        try
        {
            _in.close();
        }
        catch (Exception exe) {}
        super.notifyTransferSuccessful();

    } // end: notifyTransferSuccessful


    /**
     * Notifies observers when the transfer is terminated.
     *
     * @param ex            the exception that terminated the transfer
     */
    public void notifyTransferTerminated(TransferAdapterException ex)
    {
        try
        {
            _in.close();
        }
        catch (Exception exe) {}
        _in = null;
        super.notifyTransferTerminated(ex);

    } // end: notifyTransferTerminated


    /**
     * Sends the data from the input stream to the server, where it
     * will be written to a file identified by fileName in the current working
     * directory of the server.  If the file already exists on the server, it
     * will be overwritten if permitted.
     *
     * If the offset is non-zero, the contents of that file will be written to
     * the server from that offset.  It is assumed that the input stream
     * has already been set to the desired offset.
     *
     * @exception TransferAdapterException  an error occured that terminated
     *                                      the file transfer
     */
    abstract public void putFile() throws TransferAdapterException;


    /**
     * This method is run when the thread holding it is started.
     */
    public void run()
    {
        try
        {
            putFile();
            this.notifyTransferSuccessful();
        }
        catch (TransferAdapterException tae)
        {
            this.notifyTransferTerminated(tae);
        }

    } // end: run


    /**
     * Mutator for the input stream.
     *
     * @param in            the input stream
     *
     */
    public void setInputStream(InputStream in)
    {
        _in = in;

    } // end: setInputStream


} // end: FileSender
