/*
 * @(#)FileTransfer.java	1.0 2001/03/12
 *
 * Copyright (C) 2001 by Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter;


// Java imports
import java.util.Enumeration;
import java.util.Vector;

import net.sourceforge.journeyftp.adapter.resources.AdapterResourceBundle;


/**
 * Abstract class enabling asynchronous processing of file transfers by
 * <code>TransferAdapter</code> objects.  It relies on the adapter to
 * create an appropriate <code>FileTransferConnection</code> object to allow
 * the transfer to take place.
 *
 * @author  Maru Newby
 * @version 1.0, 2001/03/12
 */
abstract public class FileTransfer implements FileTransferObservable, Runnable
{
    // TODO: remove duplication (AbstractTransferAdapter)
    public final static String ERR_ADAPTER_OPERATION_FAILED =
                                               "Adapter.OperationFailed";


    // Instance variables
    private boolean                _abort      = false;
    private FileTransferConnection _connection = null;
    private String                 _fileName   = "";
    private Vector                 _observers  = new Vector();
    private long                   _offset     = 0;
    private VirtualFile            _file       = null;


    /**
     * Default constructor initializes the connection object to null.
     */
    public FileTransfer()
    {
        this(null);
    }


    /**
     * Constructor initializing the connection to that provided.
     *
     * @param connection    the transfer connection that will be used
     *                      to transfer the file
     */
    public FileTransfer(FileTransferConnection connection)
    {
        setConnection(connection);
    }


    /**
     * Stops the current file transfer.
     */
    public void abortTransfer()
    {
        _abort = true;
    }


    /**
     * Adds an observer.
     *
     * @param observer      the observer to add
     */
    public void addObserver(FileTransferObserver observer)
    {
        _observers.add(observer);

    } // end: addObserver


    /**
     * Accessor for the connection.
     *
     * @return              the connection used to transfer files
     *
     */
    public FileTransferConnection getConnection()
    {
        return ( _connection );

    } // end: getConnection


    /**
     * Accessor for the file name.
     *
     * @return              the file name
     */
    public String getFileName()
    {
        return (_fileName);

    } // end: getFileName


    /**
     * Accessor for the offset.
     *
     * @return              the offset
     */
    public long getOffset()
    {
        return (_offset);

    } // end: getOffset


    /**
     * Retrieves a resource string from the default bundle.
     *
     * @param key           the key of the resource string
     * @return              the resource string
     */
    protected String getString(String key)
    {
        return (AdapterResourceBundle.getResourceString(key));

    } // end: getString


    /**
     * Retrieves a resource string from the default bundle and substitutes the
     * given parameter.
     *
     * @param key           the key of the resource string
     * @param param1        a substitution parameter
     * @return              the resource string
     */
    protected String getString(String key, String param1)
    {
        return (AdapterResourceBundle.getResourceString(key,
                                                     new String[] { param1 }));

    } // end: getString


    /**
     * Accessor for the virtual file.
     *
     * @return              the virtual file
     */
    public VirtualFile getVirtualFile()
    {
        return (_file);

    } // end: getVirtualFile


    /**
     * Notifies observers when bytes are transferred.
     *
     * @param bytes         the number of bytes transferred
     */
    public void notifyDataTransferred(long bytes)
    {
        Enumeration enum = _observers.elements();
        while (enum.hasMoreElements())
           ((FileTransferObserver)enum.nextElement()).dataTransferred(bytes);

    } // end: notifyDataRecieved


    /**
     * Notifies observers when the transfer is completed successfully.
     */
    public void notifyTransferSuccessful()
    {
        Enumeration enum = _observers.elements();
        while (enum.hasMoreElements())
           ((FileTransferObserver)enum.nextElement()).transferSuccessful();

    } // end: notifyTransferSuccessful


    /**
     * Notifies observers when the transfer is terminated.
     *
     * @param ex            the exception that terminated the transfer
     */
    public void notifyTransferTerminated(TransferAdapterException ex)
    {
        Enumeration enum = _observers.elements();
        while (enum.hasMoreElements())
           ((FileTransferObserver)enum.nextElement()).transferTerminated(ex);

    } // end: notifyTransferTerminated


    /**
     * Helper method to throw the operation failed message.
     *
     * @param operation     the operation being attempted
     * @param ex            the exception that failed the operation
     * @parma msg           the error message describing the operation failure
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    public void operationFailed(String operation, Exception ex, String msg)
                                              throws OperationFailedException
    {
        throw (new OperationFailedException(
                getString(ERR_ADAPTER_OPERATION_FAILED,
                          operation),
                ex,
                msg));

    } // end: operationFailed


    /**
     * Removes an observer.
     *
     * @param observer      the observer to remove
     */
    public void removeObserver(FileTransferObserver observer)
    {
        _observers.remove(observer);

    } // end: removeObserver


    /**
     * Mutator for the connection.
     *
     * @param connection    the connection used to transfer files
     */
    public void setConnection(FileTransferConnection connection)
    {
        _connection = connection;

    } // end: setConnection


    /**
     * Mutator for the file name.
     *
     * @param fileName      the file name
     */
    public void setFileName(String fileName)
    {
        _fileName = fileName;

    } // end: setFileName


    /**
     * Mutator for the offset.
     *
     * @param offset        the offset
     */
    public void setOffset(long offset)
    {
        _offset = offset;

    } // end: getOffset


    /**
     * Mutator for the virtual file
     *
     * @param file        the virtual file
     */
    public void setVirtualFile(VirtualFile file)
    {
        _file = file;

    } // end: setVirtualFile



    /**
     * Indicates whether or not a request to abort the transfer was made.
     *
     * @return              whether or not to abort the transfer
     */
    public boolean transferAbortRequested()
    {
        return (_abort);

    } // end: transferAbortRequested


} // end: FileTransfer
