/*
 * @(#)TransferAdapter.java	1.0 2000/03/05
 *
 * Copyright (C) 2001 by Maru Newby, David Tarc
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter;


// Java Imports
import java.io.InputStream;
import java.io.OutputStream;


/**
 * This interface defines the contractual obligations of all journey ftp
 * transfer adapters.
 *
 * @author  Maru Newby, David Tarc
 * @version 1.0, 2000/03/05
 */
public interface TransferAdapter extends FileTransferObservable
{


    /**
     * Opens a connection to a server using the provided
     * <code>ConnectionInfo</code> object.  If no exception is thrown, a
     * connection was successfully made.
     *
     * @param info          the <code>ConnectionInfo</code> object
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception ConnectionRefusedException    the protocol was unable to
     *                                          negotiate a connection with the
     *                                          server
     * @exception HostNotFoundException         the server name could not be
     *                                          resolved
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception PermissionDeniedException     the protocol was unable to
     *                                          authenticate itself with the
     *                                          server
     */
    public void connect(ConnectionInfo info) throws ConnectionFailedException,
                                                    ConnectionRefusedException,
                                                    HostNotFoundException,
                                                    OperationFailedException,
                                                    PermissionDeniedException;


    /**
     * Creates a directory in the current working directory of the server.
     *
     * @param dirName       the name of the directory to create
     *
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception DirectoryExistsException      a directory with the same name
     *                                          already exists
     * @exception FileExistsException           a file with the same name
     *                                          already exists
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception PermissionDeniedException     the specified directory could
     *                                          not be created with the
     *                                          provided credentials
     */
    public void createDirectory(String dirName)
                                throws ConnectionFailedException,
                                       DirectoryExistsException,
                                       FileExistsException,
                                       OperationFailedException,
                                       PermissionDeniedException;


    /**
     * Disconnects from the server.
     */
    public void disconnect();


    /**
     * Retrieves the identified filename from the current working directory of
     * the server and writes it to the given output stream.
     *
     * If the offset is non-zero, the contents of that file will be written to
     * the output stream from that offset.  It is assumed that the output stream
     * has already been set to the desired offset.
     *
     * Note that this method is designed to be asynchronous - using threads
     * and the <code>FileTransfer</code> object it should initiate a transfer
     * and communicate via the <code>TransferObserver</code> interface.
     *
     * @param file          the <code>VirtualFile</code> pointing to the file
     *                      to retrieve
     * @param out           the output stream to write the file to
     * @param offset        the offset, in bytes, to read the file from
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    public void getFile(VirtualFile file, OutputStream out, long offset)
                        throws ConnectionFailedException,
                               OperationFailedException;


    /**
     * Retrieves the path of the current working directory of the server.
     *
     * @return              the path of the server's current working directory
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    public String getWorkingDirectory() throws ConnectionFailedException,
                                               OperationFailedException;


    /**
     * Indicates whether or not the adapter is currently connected.
     *
     * @return              whether the adapter is currently connected
     */
    public boolean isConnected();


    /**
     * Indicates whether the adapter is currently transfering a file or not.
     *
     * @return              whether the adapter is currently transferring a file
     */
    public boolean isTransferInProgress();


    /**
     * Returns an array of <code>VirtualFile</code> objects, representing the
     * files and directories contained in the current working directory of the
     * server.
     *
     * @return              the array of <code>VirtualFile</code> objects
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception ResourceNotFoundException     the working directory does not
     *                                          contain any files or the list
     *                                          cannot be parsed
     */
    public VirtualFile[] list() throws ConnectionFailedException,
                                       OperationFailedException,
                                       ResourceNotFoundException;


    /**
     * Returns a <code>VirtualFile</code> object, representing the specified
     * file.
     *
     * @return              a <code>VirtualFile</code> object of the desired
     *                      file
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception IncorrectFileTypeException    a directory was specified for
     *                                          listing
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception ResourceNotFoundException     the identified file does not
     *                                          exist
     */
    public VirtualFile listFile(String fileName)
                                throws ConnectionFailedException,
                                       IncorrectFileTypeException,
                                       OperationFailedException,
                                       ResourceNotFoundException;


    /**
     * Sends the data from the provided input stream to the server, where it
     * will be written to a file identified by fileName in the current working
     * directory of the server.  If the file already exists on the server, it
     * will be overwritten if permitted.
     *
     * If the offset is non-zero, the contents of that file will be written to
     * the server from that offset.  It is assumed that the input stream
     * has already been set to the desired offset.
     *
     * Note that this method is designed to be asynchronous - using threads
     * and the <code>FileTransfer</code> object it should initiate a transfer
     * and communicate via the <code>TransferObserver</code> interface.
     *
     * @param fileName      the name of the file to be sent to the server
     * @param in            the input stream to read the file from
     * @param offset        the offset, in bytes, to write the file from
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    public void putFile(String fileName, InputStream in, long offset)
                        throws ConnectionFailedException,
                               OperationFailedException;


    /**
     * Removes the specified file/dir from the server.
     *
     * @param file          the <code>VirtualFile</code> object identifying
     *                      the remote file/dir to remove
     * @exception ConnectionFailedException     an connection-related error
     *                                          prevented successful completion
     *                                          of the operation
     * @exception DirectoryNotEmptyException    the specified directory was not
     *                                          empty
     * @exception IncorrectFileTypeException    a file or directory was
     *                                          specified for removal but
     *                                          type was wrong
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception PermissionDeniedException     the specified file could
     *                                          not be removed with the
     *                                          provided credentials
     * @exception ResourceNotFoundException     the file targeted for
     *                                          removal could not be found
     */
    public void remove(VirtualFile file) throws ConnectionFailedException,
                                                DirectoryNotEmptyException,
                                                IncorrectFileTypeException,
                                                OperationFailedException,
                                                PermissionDeniedException,
                                                ResourceNotFoundException;


    /**
     * Renames the specified resource.
     *
     * @param resourceName  the name of the resource to rename
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception DirectoryExistsException      a directory with the new name
     *                                          already exists
     * @exception FileExistsException           a file with the new name
     *                                          already exists
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception PermissionDeniedException     the specified resource could
     *                                          not be renamed with the
     *                                          provided credentials
     * @exception ResourceNotFoundException     the resource targeted for
     *                                          renaming could not be found
     */
    public void rename(String resourceName, String newName)
                       throws ConnectionFailedException,
                              DirectoryExistsException,
                              FileExistsException,
                              OperationFailedException,
                              PermissionDeniedException,
                              ResourceNotFoundException;


    /**
     * Sets the current working directory on the server.
     *
     * @param path          the path of the new working directory
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception IncorrectFileTypeException
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception ResourceNotFoundException     the specified path does not
     *                                          exist or is unavailable on the
     *                                          server
     * @exception PermissionDeniedException     the specified path cannot be
     *                                          accessed with the provided
     *                                          credentials
     */
    public void setWorkingDirectory(String path)
                                    throws ConnectionFailedException,
                                           IncorrectFileTypeException,
                                           OperationFailedException,
                                           PermissionDeniedException,
                                           ResourceNotFoundException;


} // end: TransferAdapter

