/*
 * @(#)TransferAdapter.java	1.0 2000/03/05
 *
 * Copyright (C) 2001 by Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter;


// Java Imports
import java.util.Enumeration;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Vector;
import java.lang.reflect.Constructor;


/**
 * Enumerates and creates transfer adapters from definitions stored in
 * external property files.
 *
 * @author   Maru Newby
 * @version  1.0, 2001/03/12
 */
public class TransferAdapterFactory
{
    // Class variables
    protected final static String BUNDLE_NAME =
                 "net.sourceforge.journeyftp.adapter.resources.AdapterList";

    protected static TransferAdapterFactory _instance = null;

    // Instance variables
    protected ResourceBundle _bundle = null;


    /**
     * Private constructor, supports singleton pattern.
     */
    protected TransferAdapterFactory()
    {
       _bundle = ResourceBundle.getBundle(BUNDLE_NAME);
    }


    /**
     * Creates the transfer adapter associated with the given identifier
     * string.
     *
     * @param identifier    the identifier for the desired transfer adapter
     * @exception UnknownAdapterException       if the identifier cannot be
     *                                          mapped to an adapter
     */
    public TransferAdapter createTransferAdapter(TransferAdapterInfo info)
                                                 throws AdapterCreationException
    {
        return (createTransferAdapter(info.getClassName()));
    } // end: createTransferAdapter


    /**
     * Creates the transfer adapter from the given class name.
     *
     * @param className     the class name of the adapter
     * @exception AdapterCreationException  if the provided class could not be
     *                                      created or did not implement the
     *                                      <code>TransferAdapter</code>
     *                                      interface
     */
    public TransferAdapter createTransferAdapter(String className)
                                                 throws AdapterCreationException
    {
        try
        {
            // Retrieve the adapter class
            Class adapterClass = Class.forName( className );

            // Pass the command key to the command object
            Constructor constructor = adapterClass.getConstructor( null );
            TransferAdapter transferAdapter =
                                 (TransferAdapter)constructor.newInstance(null);
            return ( transferAdapter );
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            throw new AdapterCreationException( ex.getMessage() );
        }

    } // end: createTransferAdapter


    /**
     * Generates an array of adapter info objects.  This enables dynamic
     * choosing a
     */
    public TransferAdapterInfo[] getAdapterList() throws ListFormatException
    {
        Vector adapterInfo = new Vector();

        try
        {
            // Get the keys from the property file
            Enumeration keys = _bundle.getKeys();

            String key;
            String value;
            StringTokenizer tokenizer;

            // Add all entries to the vector
            // Note the dereferencing to ensure garbage collection
            while (keys.hasMoreElements())
            {
                key = (String)keys.nextElement();
                value = _bundle.getString( key );
                if (! value.equals(""))
                {
                    tokenizer = new StringTokenizer(value, ",");
                    TransferAdapterInfo temp =
                               new TransferAdapterInfo( tokenizer.nextToken(),
                                                        tokenizer.nextToken(),
                                                        Integer.parseInt(key) );
                    adapterInfo.addElement(temp);
                    temp = null;
                    tokenizer = null;
                }
                key = null;
                value = null;
            }
        }
        catch (Exception ex)
        {
            throw new ListFormatException();
        }

        TransferAdapterInfo[] infoArray =
                                    new TransferAdapterInfo[adapterInfo.size()];
        adapterInfo.copyInto(infoArray);

        return (infoArray);

    } // end: getAdapterList


    /**
     * Retrieves the singleton instance of the adapter factory.
     *
     * @return              the singleton instance of the adapter factory
     */
    public static TransferAdapterFactory getInstance()
    {
        if ( _instance == null )
            _instance = new TransferAdapterFactory();

        return ( _instance );

    } // end: getInstance


} // end: TransferAdapterFactory


