/*
 * @(#)FTPAdapter.java	1.0 2001/03/12
 *
 * Copyright (C) 2001 by David Tarc, Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter.fooftp;


// Java imports
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Vector;
import java.net.InetAddress;
import java.net.UnknownHostException;

// Fooware imports
import com.fooware.net.FtpClient;
import com.fooware.net.FtpReader;
import com.fooware.net.FtpResponse;

// Adapter imports
import net.sourceforge.journeyftp.adapter.*;


/**
 * Fooware FTP implementation of a journeyftp <code>TransferAdapter</code>.
 *
 * @author  David Tarc, Maru Newby
 * @version 1.0, 2001/03/15
 */
public class FTPAdapter extends AbstractTransferAdapter
{
    // Instance variables
    private FtpClient _client    = new FtpClient();


    /**
     * Default no-arg constructor.
     */
    public FTPAdapter()
    {
        super();
    }


    /**
     * Opens a connection to a server using the provided
     * <code>ConnectionInfo</code> object.  If no exception is thrown, a
     * connection was successfully made.
     *
     * @param info          the <code>ConnectionInfo</code> object
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception ConnectionRefusedException    the protocol was unable to
     *                                          negotiate a connection with the
     *                                          server
     * @exception HostNotFoundException         the server name could not be
     *                                          resolved
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception PermissionDeniedException     the protocol was unable to
     *                                          authenticate itself with the
     *                                          server
     */
    public void connect(ConnectionInfo info) throws ConnectionFailedException,
                                                    ConnectionRefusedException,
                                                    HostNotFoundException,
                                                    OperationFailedException,
                                                    PermissionDeniedException
    {
        FtpResponse response = null;

        try
        {
            // Do host lookup first
            try
            {
                InetAddress.getByName(info.getHost());

            }
            catch (UnknownHostException uhe)
            {
                throw (new HostNotFoundException(
                                          getString(ERR_CONNECT_HOST_NOT_FOUND,
                                                    info.getHost())));
            }

            // Try to connect
            boolean connected = false;
            try
            {
                _client.connect(info.getHost());
                response = _client.getResponse();
                connected = true;
            }
            catch (IOException ioi)
            {
                // Ignore
            }

            // Connection refused
            if (! connected || ! response.isPositiveCompletion())
                throw (new ConnectionRefusedException(
                                      getString(ERR_CONNECT_CONNECTION_REFUSED,
                                                info.getHost())));

            // Login with the username
            _client.userName(info.getUsername());
            response = _client.getResponse();
            if ( !response.isPositiveIntermediary() )
                throw (new ConnectionRefusedException(
                                      getString(ERR_CONNECT_CONNECTION_REFUSED,
                                                info.getHost())));

            // If the username was accepted, send the password
            _client.password(info.getPassword());
            response = _client.getResponse();
            if ( !response.isPositiveCompletion() )
                throw (new PermissionDeniedException(
                                      getString(ERR_CONNECT_PERMISSION_DENIED,
                                                info.getHost())));
        }
        catch (IOException ioe)
        {
            operationFailed(OP_CONNECT, ioe);
        }

    } // end: connect


    /**
     * Creates a directory in the current working directory of the server.
     *
     * @param dirName       the name of the directory to create
     *
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception DirectoryExistsException      a directory with the same name
     *                                          already exists
     * @exception FileExistsException           a file with the same name
     *                                          already exists
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception PermissionDeniedException     the specified directory could
     *                                          not be created with the
     *                                          provided credentials
     */
    public void createDirectory(String dirName)
                                throws ConnectionFailedException,
                                       DirectoryExistsException,
                                       FileExistsException,
                                       OperationFailedException,
                                       PermissionDeniedException
    {
        // Verify that a directory or file with the same name does not exist
        checkDirectoryExists(dirName);

        try
        {
            // Try to create the directory
            _client.makeDirectory( dirName );
            FtpResponse response = _client.getResponse();
            if (! response.isPositiveCompletion())
                if (response.getMessage().indexOf(SH_FILE_EXISTS) != -1)
                    throw (new FileExistsException(
                                        getString(ERR_MKDIR_FILE_EXISTS,
                                                  dirName)));
                // Default all other failures to permission errors
                else
                    throw (new PermissionDeniedException(
                                         getString(ERR_MKDIR_PERMISSION_DENIED,
                                                   dirName)));
        }
        catch (IOException ioe)
        {
            operationFailed(OP_MKDIR, ioe);
        }

    } // end: createDirectory


    /**
     * Disconnects from the server.
     */
    public void disconnect()
    {
        try
        {
            _client.logout();
        }
        catch (IOException ioe)
        {
            // Ignore
        }

    } // end: disconnect


    /**
     * Returns a prepared data connection for file transfers.
     *
     * @return              the data connection
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    protected FileTransferConnection getConnection()
                                              throws ConnectionFailedException,
                                                     OperationFailedException
    {
        // Prepare for a file transfer
        try
        {
            // Set the transfer type
            _client.representationType('L', '8');
            FtpResponse response = _client.getResponse();
            if (! response.isPositiveCompletion())
                operationFailed(OP_XFER_SETUP, response.getMessage());

            // Set the transfer type
            response = _client.getResponse();
            if (! response.isPositiveCompletion())
                operationFailed(OP_XFER_SETUP, response.getMessage());

            // Set the data port
            _client.dataPort();
            response = _client.getResponse();
            if (! response.isPositiveCompletion())
                operationFailed(OP_XFER_SETUP, response.getMessage());
        }
        catch (IOException ioe)
        {
            operationFailed(OP_XFER_SETUP, ioe);
        }

        return (new FTPFileTransferConnection(_client));

    } // end: getConnection


    /**
     * Returns an uninitialied <code>FileReceiver</code> object to transfer
     * a file from the server.
     *
     * @return              the file receiver
     */
    protected FileReceiver getFileReceiver()
    {
        return (new FTPFileReceiver());

    } // end: getFileReceiver


    /**
     * Returns an uninitialied <code>FileSender</code> object to transfer
     * a file to the server.
     *
     * @return              the file sender
     */
    protected FileSender getFileSender()
    {
        return (new FTPFileSender());

    } // end: getFileSender


    /**
     * Retrieves the path of the current working directory of the server.
     *
     * @return              the path of the server's current working directory
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    public String getWorkingDirectory() throws ConnectionFailedException,
                                               OperationFailedException
    {
        try
        {
            _client.printWorkingDirectory();
        }
        catch (IOException ioe)
        {
            operationFailed(OP_PWD, ioe);
        }

        FtpResponse response = _client.getResponse();
        if (! response.isPositiveCompletion())
            operationFailed(OP_PWD, response.getMessage());

        String dir = "";
        // Wrap string parsing in error handler
        try
        {
            dir = _client.getResponse().getMessage();
            dir = dir.substring(dir.indexOf("\"") + 1, dir.lastIndexOf("\""));
        }
        catch (Exception ex)
        {
            operationFailed(OP_PWD, ex);
        }

        return ( dir );

    } // end: getWorkingDirectory


    /**
     * Indicates whether or not the adapter is currently connected.
     *
     * @return              whether the adapter is currently connected
     */
    public boolean isConnected()
    {
        try
        {
            // Perform a no-op to test connection
            _client.noOp();
        }
        catch (IOException ioe)
        {
            return (false);
        }

        return (true);

    } // end: isConnected


    /**
     * Returns an array of <code>VirtualFile</code> objects, representing the
     * files and directories contained in the current working directory of the
     * server.
     *
     * @param fileName      the name of the file to list.  If not provided, the
     *                      current working directory will be listed
     * @return              the array of <code>VirtualFile</code> objects
     * @exception ConnectionFailedException     an connection-related error
     *                                          prevented successful completion
     *                                          of the operation
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception ResourceNotFoundException     the working directory does not
     *                                          contain any files or the list
     *                                          cannot be parsed
     */
     protected VirtualFile[] list(String fileName)
                                  throws ConnectionFailedException,
                                         OperationFailedException,
                                         ResourceNotFoundException
    {
        FtpResponse response;
        FtpReader ftpIn;
        VirtualFile[] dirList = null;

        try
        {
            // set type to ASCII
            _client.representationType(FtpClient.ASCII_TYPE);
            response = _client.getResponse();
            if ( ! response.isPositiveCompletion() )
                operationFailed(OP_LIST, response.getMessage());

            // create port to which server can send data
            _client.dataPort();
            response = _client.getResponse();
            if ( ! response.isPositiveCompletion() )
                operationFailed(OP_LIST, response.getMessage());

            // request listing ...
            if (fileName.equals(""))
                ftpIn = _client.list();
            else
                ftpIn = _client.list(fileName);
            response = _client.getResponse();
            if ( ! response.isPositivePreliminary() )
                operationFailed(OP_LIST, response.getMessage());

            // Process the listing into a string
            StringBuffer buff = new StringBuffer();
            while (ftpIn.ready())
                buff.append(ftpIn.readLine() + "\n");

            // explicit close to let FtpClient know we're done
            ftpIn.close();
            response = _client.getResponse();
            response = _client.getResponse();
            if (! response.isPositiveCompletion())
                operationFailed(OP_LIST, response.getMessage());

            // Parse the results of the command
            dirList = parseList(buff.toString());

            if (dirList.length == 0)
                throw (new ResourceNotFoundException(
                                      getString(ERR_LIST_RESOURCE_NOT_FOUND)));
        }
        catch (IOException ioe)
        {
            operationFailed(OP_LIST, ioe);
        }
        catch (ListFormatException lfe)
        {
            throw (new ResourceNotFoundException(
                                         getString(ERR_LIST_PARSE_ERROR)));
        }

        return (dirList);

    } // end: list


    /**
     * Removes the specified file/dir from the server.
     *
     * @param file          the <code>VirtualFile</code> object identifying
     *                      the remote file/dir to remove
     * @exception ConnectionFailedException     an connection-related error
     *                                          prevented successful completion
     *                                          of the operation
     * @exception DirectoryNotEmptyException    the specified directory was not
     *                                          empty
     * @exception IncorrectFileTypeException    a file or directory was
     *                                          specified for removal but
     *                                          type was wrong
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception PermissionDeniedException     the specified file could
     *                                          not be removed with the
     *                                          provided credentials
     * @exception ResourceNotFoundException     the file targeted for
     *                                          removal could not be found
     */
    public void remove(VirtualFile file) throws ConnectionFailedException,
                                                DirectoryNotEmptyException,
                                                IncorrectFileTypeException,
                                                OperationFailedException,
                                                PermissionDeniedException,
                                                ResourceNotFoundException
    {
        try
        {
            String fileName = file.getName();
            if (file.isDirectory())
                _client.removeDirectory(fileName);
            else
                _client.delete(fileName);

            FtpResponse response = _client.getResponse();
            if (! response.isPositiveCompletion())
            {
                String error = response.getMessage();
                if (error.indexOf(SH_NO_SUCH_FILE) != -1)
                    throw (new ResourceNotFoundException(
                                     getString(ERR_REMOVE_RESOURCE_NOT_FOUND,
                                               fileName)));
                else if (error.indexOf(SH_PERMISSION_DENIED) != -1)
                    throw (new PermissionDeniedException(
                                     getString(ERR_REMOVE_PERMISSION_DENIED,
                                                fileName)));
                else if (error.indexOf(SH_DIR_NOT_EMPTY) != -1)
                    throw (new PermissionDeniedException(
                                     getString(ERR_REMOVE_DIR_NOT_EMPTY,
                                               fileName)));
                else
                    operationFailed(OP_CWD, error);
            }
        }
        catch (IOException ioe)
        {
            operationFailed(OP_REMOVE, ioe);
        }

    } // end: remove


    /**
     * Renames a file or directory
     *
     * @param     file       the file or directory to be renamed
     * @param     newName    the name the file will be renamed to
     * @param     overwrite  if newName already exists, setting overwrite to
     *                       true will cause it to be deleted
     * @result    boolean    true if file was renamed, false otherwise
     * @exception ConnectionFailedException    the connection to the server
     *                                         was terminated or did not exist
     * @exception PermissionDeniedException    the user did not have permission
     *                                         to remove the file or directory
     * @exception ResourceNotFoundException    the file or directory
     *                                         does not exist
     */
    public void rename( String fileName, String newName )
                           throws ConnectionFailedException,
                                  PermissionDeniedException,
                                  ResourceNotFoundException,
                                  FileExistsException,
                                  DirectoryExistsException,
                                  OperationFailedException
    {
        throw (new OperationFailedException(
                                getString(ERR_ADAPTER_OPERATION_NOT_SUPPORTED,
                                          OP_RENAME),
                                null,
                                ""));
    } // end: rename


    /**
     * Sets the current working directory on the server.
     *
     * @param path          the path of the new working directory
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception IncorrectFileTypeException
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception ResourceNotFoundException     the specified path does not
     *                                          exist or is unavailable on the
     *                                          server
     * @exception PermissionDeniedException     the specified path cannot be
     *                                          accessed with the provided
     *                                          credentials
     */
    public void setWorkingDirectory(String path)
                                    throws ConnectionFailedException,
                                           IncorrectFileTypeException,
                                           OperationFailedException,
                                           PermissionDeniedException,
                                           ResourceNotFoundException
    {
        try
        {
            _client.changeWorkingDirectory( path );
            FtpResponse response = _client.getResponse();
            if (! response.isPositiveCompletion())
            {
                String error = response.getMessage();
                if (error.indexOf(SH_NO_SUCH_FILE) != -1)
                    throw (new ResourceNotFoundException(
                                     getString(ERR_CWD_RESOURCE_NOT_FOUND,
                                               path)));
                else if (error.indexOf(SH_NOT_A_DIRECTORY) != -1)
                    throw (new IncorrectFileTypeException(
                                     getString(ERR_CWD_INCORRECT_FILE_TYPE,
                                                path)));
                else if (error.indexOf(SH_PERMISSION_DENIED) != -1)
                    throw (new PermissionDeniedException(
                                     getString(ERR_CWD_PERMISSION_DENIED,
                                                path)));
                else
                    operationFailed(OP_CWD, error);
            }
        }
        catch (IOException ioe)
        {
            operationFailed(OP_CWD, ioe);
        }

    } // end: setWorkingDirectory


} // end: FTPAdapter

