/*
 * @(#)FTPFileReceiver.java	1.0 2001/03/12
 *
 * Copyright (C) 2001 by David Tarc, Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter.fooftp;


// Java imports
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

// Fooware imports
import com.fooware.net.FtpClient;
import com.fooware.net.FtpReader;
import com.fooware.net.FtpResponse;

// Adapter imports
import net.sourceforge.journeyftp.adapter.*;


/**
 * FTP implementation of asynchronous ftp file receiver.
 *
 * @author  David Tarc, Maru Newby
 * @version 1.0, 2001/03/15
 */
public class FTPFileReceiver extends FileReceiver
{
    // Class variables
    public static final int BUFF_SIZE = 8192;


    /**
     * Retrieves the identified filename from the current working directory of
     * the server and writes it to an output stream.
     *
     * If the offset is non-zero, the contents of that file will be written to
     * the output stream from that offset.  It is assumed that the output stream
     * has already been set to the desired offset.
     *
     * @exception TransferAdapterException  an error occured that terminated
     *                                      the file transfer
     */
    public void getFile() throws TransferAdapterException
    {
        char[] buffer = new char[BUFF_SIZE];
        int charsRead;
        FtpResponse response;

        // Retrieve the parameters
        OutputStream out = this.getOutputStream();
        String fileName = this.getFileName();
        long offset = this.getOffset();

        // Retrieve the client object
        FTPFileTransferConnection connection =
                                (FTPFileTransferConnection)this.getConnection();
        FtpClient client = connection.getFtpClient();

        VirtualFile file = getVirtualFile();

        // Throw an error if the offset it nonzero
        if (offset > 0 )
            throw (new ResumeNotSupportedException(
           getString(AbstractTransferAdapter.ERR_GET_FILE_RESUME_NOT_SUPPORTED,
                     fileName)));

        try
        {
            // Make a file retrieval request to the server
            FtpReader ftpIn = client.retrieve(fileName);
            response = client.getResponse();
            if (! response.isPositivePreliminary())
                throw new TransferAdapterException(response.getMessage());

            // Create a writer for the output stream
            BufferedWriter bufferedOut = new BufferedWriter(
                                                   new OutputStreamWriter(out));

            if (!file.isDirectory() && file.getSize() == 0)
            {
                // Write out all the data, taking care to notify observers
                // of the bytes transferred.
                while ((! this.transferAbortRequested()) &&
                       ((charsRead = ftpIn.read(buffer, 0, buffer.length)) != -1))
                {
                    bufferedOut.write(buffer, 0, charsRead);
                    this.notifyDataTransferred(charsRead);
                }
            }

            // Won't work without this
            bufferedOut.flush();
            bufferedOut.close();

            // Check for an abort condition
            if (this.transferAbortRequested())
            {
                client.abort();
                response = client.getResponse();

                // If the transfer was aborted
                if (! response.isPositiveCompletion())
                {
                    // Prepare to indicate an aborted transfer
                    TransferAbortedException tae =
                            new TransferAbortedException(response.getMessage());
                    // Read the positive response for the abort command
                    response = client.getResponse();
                    throw (tae);
                }
            }

            // Close the stream and check for completion message
            ftpIn.close();
            response = client.getResponse();
            if (! response.isPositiveCompletion())
                throw new TransferAdapterException(response.getMessage());

            // If no exception is thrown before here, transfer was successful
        }
        catch (IOException ioe)
        {
            throw (new ConnectionFailedException(ioe.getMessage()));
        }

    } // end: getFile


} // end: FTPFileReceiver