/*
 * @(#)TelnetAdapter.java	1.0 2001/03/21
 *
 * Copyright (C) 2001 by Mark Cunningham, Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter.jta;


// Java imports
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.UnknownHostException;

// Adapter imports
import net.sourceforge.journeyftp.adapter.*;


/**
 * JTA telnet implementation of a journeyftp <code>TransferAdapter</code>.
 *
 * @author  Mark Cunningham, Maru Newby
 * @version 1.0, 2001/03/21
 */
public class TelnetAdapter extends AbstractTransferAdapter
{
    // Class variables
    // Don't use @ or $ - prompt has to work for csh/ksh/bash
    protected final static String DEFAULT_PROMPT = "[login:telnet]%";

    // Instance variables
    private TelnetClient _client = new TelnetClient();


    /**
     * Default no-arg constructor.
     */
    public TelnetAdapter()
    {
        super();
    }


    /**
     * Opens a connection to a server using the provided
     * <code>ConnectionInfo</code> object.  If no exception is thrown, a
     * connection was successfully made.
     *
     * @param info          the <code>ConnectionInfo</code> object
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception ConnectionRefusedException    the protocol was unable to
     *                                          negotiate a connection with the
     *                                          server
     * @exception HostNotFoundException         the server name could not be
     *                                          resolved
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception PermissionDeniedException     the protocol was unable to
     *                                          authenticate itself with the
     *                                          server
     */
    public void connect(ConnectionInfo info) throws ConnectionFailedException,
                                                    ConnectionRefusedException,
                                                    HostNotFoundException,
                                                    OperationFailedException,
                                                    PermissionDeniedException
    {
        try
        {
            // Try to connect
            _client.connect(info.getHost(), 23);
        }
        catch (IOException ioe)
        {
            if (ioe instanceof UnknownHostException)
                throw (new HostNotFoundException(
                                          getString(ERR_CONNECT_HOST_NOT_FOUND,
                                                    info.getHost())));
            else
                throw (new ConnectionRefusedException(
                                      getString(ERR_CONNECT_CONNECTION_REFUSED,
                                                info.getHost())));
        }

        try
        {
            // Login with the username and password
            _client.login(info.getUsername(), info.getPassword());

            // Read in the first message
            String welcome = _client.readResponse();
            // If the length is zero, probably failed login
            if (welcome.length() == 0)
            {
                // Read in the prompt
                String prompt = _client.readResponse();

                // If the prompt indicate an incorrect login
                if (prompt.indexOf(SH_LOGIN_INCORRECT) != -1)
                {
                    disconnect();
                    throw (new PermissionDeniedException(
                                       getString(ERR_CONNECT_PERMISSION_DENIED,
                                                 info.getHost())));
                }
            }

            // Determine the appropriate prompt
            try
            {
                _client.negotiatePrompt(DEFAULT_PROMPT);
            }
            catch (Exception ex)
            {
                // Disconnect and fail the connection attempt
                disconnect();
                throw (new ConnectionFailedException(
                                  getString(this.ERR_CONNECT_CONNECTION_FAILED,
                                            info.getHost())));
            }

        }
        catch (IOException ioe)
        {
            operationFailed(OP_CONNECT, ioe);
        }

    } // end: connect


    /**
     * Creates a directory in the current working directory of the server.
     *
     * @param dirName       the name of the directory to create
     *
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception DirectoryExistsException      a directory with the same name
     *                                          already exists
     * @exception FileExistsException           a file with the same name
     *                                          already exists
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception PermissionDeniedException     the specified directory could
     *                                          not be created with the
     *                                          provided credentials
     */
    public void createDirectory(String dirName)
                                throws ConnectionFailedException,
                                       DirectoryExistsException,
                                       FileExistsException,
                                       OperationFailedException,
                                       PermissionDeniedException
    {
        // Verify that a directory or file with the same name does not exist
        checkDirectoryExists(dirName);

        // Attempt to create the directory
        try
        {
            TelnetResponse response = _client.sendCommand("mkdir " + dirName);
            if (! response.successfulCompletion())
            {
                // Get the first line of the response
                String error = response.getErrorLine();
                if (error.indexOf(SH_FILE_EXISTS) != -1)
                    throw (new FileExistsException(
                                        getString(ERR_MKDIR_FILE_EXISTS,
                                                  dirName)));
                // Default all other failures to permission errors
                else
                    throw (new PermissionDeniedException(
                                         getString(ERR_MKDIR_PERMISSION_DENIED,
                                                   dirName)));
            }
        }
        catch (IOException ioe)
        {
            operationFailed(OP_MKDIR, ioe);
        }

    } // end: createDirectory


    /**
     * Disconnects from the server.
     */
    public void disconnect()
    {
        try
        {
            _client.disconnect();
        }
        catch (IOException ioe)
        {
            // Ignore
        }

    } // end: disconnect


    /**
     * Returns a prepared data connection for file transfers.
     *
     * @return              the data connection
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    protected FileTransferConnection getConnection()
                                              throws ConnectionFailedException,
                                                     OperationFailedException
    {
        return (new TelnetFileTransferConnection(_client));

    } // end: getConnection


    /**
     * Returns an uninitialied <code>FileReceiver</code> object to transfer
     * a file from the server.
     *
     * @return              the file receiver
     */
    protected FileReceiver getFileReceiver()
    {
        return (new TelnetFileReceiver());

    } // end: getFileReceiver


    /**
     * Returns an uninitialied <code>FileSender</code> object to transfer
     * a file to the server.
     *
     * @return              the file sender
     * @exception OperationFailedException  the adapter is not able to provide
     *                                      the sender
     */
    protected FileSender getFileSender() throws OperationFailedException
    {
        throw (new OperationFailedException(
                                getString(ERR_ADAPTER_OPERATION_NOT_SUPPORTED,
                                          OP_PUT_FILE),
                                null,
                                ""));
    } // end: getFileSender


    /**
     * Retrieves the path of the current working directory of the server.
     *
     * @return              the path of the server's current working directory
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    public String getWorkingDirectory() throws ConnectionFailedException,
                                               OperationFailedException
    {
        TelnetResponse response = null;

        // Execute the command
        try
        {
            response = _client.sendCommand("pwd");
        }
        catch (IOException ioe)
        {
            operationFailed(OP_PWD, ioe);
        }

        return (response.getMessage());

    } // end: getWorkingDirectory


    /**
     * Indicates whether the adapter is currently transfering a file or not.
     *
     * @return              whether the adapter is currently transferring a file
     */
    public boolean isConnected()
    {
        boolean connected = false;
        // First check if a transfer is in progress
        if (isTransferInProgress())
            connected = true;
        // If not, do a pwd to check for a connection
        else
        {
            try
            {
                getWorkingDirectory();
                connected = true;
            }
            catch (TransferAdapterException tae) { }
        }

        return (connected);

    } // end: isConnected


    /**
     * Returns an array of <code>VirtualFile</code> objects, representing the
     * files and directories contained in the current working directory of the
     * server.
     *
     * @param fileName      the name of the file to list.  If not provided, the
     *                      current working directory will be listed
     * @return              the array of <code>VirtualFile</code> objects
     * @exception ConnectionFailedException     an connection-related error
     *                                          prevented successful completion
     *                                          of the operation
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception ResourceNotFoundException     the working directory does not
     *                                          contain any files or the list
     *                                          cannot be parsed
     */
     protected VirtualFile[] list(String fileName)
                                  throws ConnectionFailedException,
                                         OperationFailedException,
                                         ResourceNotFoundException
    {
        VirtualFile[] dirList = null;

        try
        {
            // Create the command
            if (fileName == null)
                fileName = "";
            // -d : List a directory as an entry, not its contents
            if (!fileName.equals(""))
                fileName = " -d " + fileName;
            // -l : long, -L : links, -A : ignore . and ..
            String cmd = "ls -l -L -A" + fileName;
            cmd = cmd.trim();

            // Execute the command
            TelnetResponse response = _client.sendCommand(cmd);

            // Check that the command returned data
            String message = response.getMessage();
            int firstLineLength = message.indexOf("\n");
            int indexOfTotal = message.indexOf("total");
            boolean telnetCodes = false;
            // Check that total occurs in the first line
            if (indexOfTotal != -1 && indexOfTotal >= firstLineLength)
                throw (new ResourceNotFoundException(
                                      getString(ERR_LIST_RESOURCE_NOT_FOUND)));
            // Test for the presence of telnet codes
            else
                telnetCodes = message.startsWith("[0m");

            // Parse the results of the command
            dirList = parseList(response.getMessage(), telnetCodes);

            if (dirList.length == 0)
                throw (new ResourceNotFoundException(
                                      getString(ERR_LIST_RESOURCE_NOT_FOUND)));
        }
        catch (ListFormatException lfe)
        {
            throw (new ResourceNotFoundException(
                                         getString(ERR_LIST_PARSE_ERROR)));
        }
        catch (IOException ioe)
        {
            operationFailed(OP_LIST, ioe);
        }

        return (dirList);

    } // end: list


    /**
     * Removes the specified file/dir from the server.
     *
     * @param file          the <code>VirtualFile</code> object identifying
     *                      the remote file/dir to remove
     * @exception ConnectionFailedException     an connection-related error
     *                                          prevented successful completion
     *                                          of the operation
     * @exception DirectoryNotEmptyException    the specified directory was not
     *                                          empty
     * @exception IncorrectFileTypeException    a file or directory was
     *                                          specified for removal but
     *                                          type was wrong
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception PermissionDeniedException     the specified file could
     *                                          not be removed with the
     *                                          provided credentials
     * @exception ResourceNotFoundException     the file targeted for
     *                                          removal could not be found
     */
    public void remove(VirtualFile file) throws ConnectionFailedException,
                                                DirectoryNotEmptyException,
                                                IncorrectFileTypeException,
                                                OperationFailedException,
                                                PermissionDeniedException,
                                                ResourceNotFoundException
    {

        try
        {
            String fileName = file.getName();
            // Different handling for directories and files
            String cmd;
            if (file.isDirectory())
                cmd = "rmdir ";
            else
                cmd = "rm ";
            TelnetResponse response = _client.sendCommand(cmd + fileName);
            if (! response.successfulCompletion())
            {
                // Get the first line of the response
                String error = response.getErrorLine();
                if (error.indexOf(SH_NO_SUCH_FILE) != -1)
                    throw (new ResourceNotFoundException(
                                     getString(ERR_REMOVE_RESOURCE_NOT_FOUND,
                                               fileName)));
                else if (error.indexOf(SH_NOT_A_DIRECTORY) != -1)
                    throw (new IncorrectFileTypeException(
                                     getString(ERR_REMOVE_INCORRECT_FILE_TYPE,
                                                fileName, "file")));
                else if (error.indexOf(SH_IS_A_DIRECTORY) != -1)
                    throw (new IncorrectFileTypeException(
                                     getString(ERR_REMOVE_INCORRECT_FILE_TYPE,
                                                fileName, "directory")));
                else if (error.indexOf(SH_PERMISSION_DENIED) != -1)
                    throw (new PermissionDeniedException(
                                     getString(ERR_REMOVE_PERMISSION_DENIED,
                                                fileName)));
                else if (error.indexOf(SH_DIR_NOT_EMPTY) != -1)
                    throw (new PermissionDeniedException(
                                     getString(ERR_REMOVE_DIR_NOT_EMPTY,
                                               fileName)));
                else
                    operationFailed(OP_CWD, error);
            }
        }
        catch (IOException ioe)
        {
            operationFailed(OP_REMOVE, ioe);
        }

    } // end: removeFile


    /**
     * Renames the specified resource.
     *
     * @param resourceName  the name of the resource to rename
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception DirectoryExistsException      a directory with the new name
     *                                          already exists
     * @exception FileExistsException           a file with the new name
     *                                          already exists
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception PermissionDeniedException     the specified resource could
     *                                          not be renamed with the
     *                                          provided credentials
     * @exception ResourceNotFoundException     the resource targeted for
     *                                          renaming could not be found
     */
    public void rename(String resourceName, String newName)
                       throws ConnectionFailedException,
                              DirectoryExistsException,
                              FileExistsException,
                              OperationFailedException,
                              PermissionDeniedException,
                              ResourceNotFoundException
    {
        try
        {
            VirtualFile file = null;
            // Check if the first file exists
            try
            {
                file = listFile(resourceName);
            }
            catch (Exception ex)
            {
                throw (new ResourceNotFoundException(
                                 getString(ERR_RENAME_RESOURCE_NOT_FOUND,
                                           resourceName)));
            }
            // Second, check if a file/dir exists with the new resource
            file = null;
            try
            {
                file = listFile(newName);
            }
            catch (Exception ex)
            {
                // Ignore - file/dir does not exist
            }
            finally
            {
                // If a resource with the new name exists
                if (file != null)
                {
                    if (file.isDirectory())
                        throw (new DirectoryExistsException(
                                     getString(ERR_RENAME_DIR_EXISTS,
                                               resourceName, newName)));
                    else
                        throw (new FileExistsException(
                                     getString(ERR_RENAME_FILE_EXISTS,
                                               resourceName, newName)));
                }
            }

            TelnetResponse response = _client.sendCommand("mv -f " +
                                                          resourceName + " " +
                                                          newName);
            if (! response.successfulCompletion())
            {
                // Get the first line of the response
                String error = response.getErrorLine();
                if (error.indexOf(SH_PERMISSION_DENIED) != -1)
                    throw (new ResourceNotFoundException(
                                     getString(ERR_RENAME_PERMISSION_DENIED,
                                               resourceName)));
                else
                    operationFailed(OP_CWD, error);
            }
        }
        catch (IOException ioe)
        {
            operationFailed(this.OP_RENAME, ioe);
        }

    } // end: rename


    /**
     * Sets the current working directory on the server.
     *
     * @param path          the path of the new working directory
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception IncorrectFileTypeException
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception ResourceNotFoundException     the specified path does not
     *                                          exist or is unavailable on the
     *                                          server
     * @exception PermissionDeniedException     the specified path cannot be
     *                                          accessed with the provided
     *                                          credentials
     */
    public void setWorkingDirectory(String path)
                                    throws ConnectionFailedException,
                                           IncorrectFileTypeException,
                                           OperationFailedException,
                                           PermissionDeniedException,
                                           ResourceNotFoundException
    {
        try
        {
            TelnetResponse response = _client.sendCommand("cd " + path);

            if (! response.successfulCompletion())
            {
                // Get the first line of the response
                String error = response.getErrorLine();
                if (error.indexOf(SH_NO_SUCH_FILE) != -1)
                    throw (new ResourceNotFoundException(
                                     getString(ERR_CWD_RESOURCE_NOT_FOUND,
                                               path)));
                else if (error.indexOf(SH_NOT_A_DIRECTORY) != -1)
                    throw (new IncorrectFileTypeException(
                                     getString(ERR_CWD_INCORRECT_FILE_TYPE,
                                                path)));
                else if (error.indexOf(SH_PERMISSION_DENIED) != -1)
                    throw (new PermissionDeniedException(
                                     getString(ERR_CWD_PERMISSION_DENIED,
                                                path)));
                else
                    operationFailed(OP_CWD, error);
            }
        }
        catch (IOException ioe)
        {
            operationFailed(OP_CWD, ioe);
        }

    } // end: setWorkingDirectory


} // end: TelnetAdapter