/*
 * @(#)TelnetClient.java	1.0 2001/03/21
 *
 * Copyright (C) 2001 by Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter.jta;


// Java imports
import java.io.*;

// JTA imports
import de.mud.telnet.TelnetWrapper;

// Adapter imports
import net.sourceforge.journeyftp.adapter.*;
import net.sourceforge.journeyftp.adapter.resources.*;


/**
 * Telnet implementation of asynchronous file receiver.
 *
 * @author  Maru Newby
 * @version 1.0, 2001/03/21
 */
public class TelnetClient extends TelnetWrapper
{
    // Instance variables
    protected String _prompt;
    private final String CANCEL = "^C";

    /**
     * Aborts a telnet transfer.
     *
     * @param oldPrompt     the prompt to restore.
     */
    public void abortTransfer(String oldPrompt)
    {
        try
        {
            setPrompt(null);
            send(CANCEL);
            setPrompt(oldPrompt);
            stabilizePrompt(oldPrompt);
        }
        catch (IOException ioe)
        {
            new OperationFailedException(
                      AdapterResourceBundle.getResourceString(
                           AbstractTransferAdapter.ERR_TRANSFER_ABORT_FAILURE),
                      ioe,
                      "");
        }

    } // end: abortTransfer


    /**
     *  Connect the socket and open the connection.
     *
     *
     */
    public void connect(String host, int port) throws IOException
    {
        try
        {
            socket = new java.net.Socket(host, port);
            in = socket.getInputStream();
            out = socket.getOutputStream();
            reset();
        } catch(Exception e) {
            disconnect();
            throw ((IOException)e);
        }

    } // end: connect


    /**
     * Convenience accessor for the telnet input stream.
     */
    public InputStream getInputStream()
    {
        return (in);

    } // end: getInputStream


    /**
     * Accessor for the current system prompt.
     *
     * @return              the current system prompt
     */
    public String getPrompt()
    {
        return (_prompt);

    } // end: getPrompt


    /**
     * Accessor for the current system prompt.
     *
     * @return              the current system prompt
     */
    public BufferedReader getTelnetReader()
    {
        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
        return (reader);

    } // end: getPrompt


    /**
     * Negotiates the prompt to be used for this telnet session.  Garbage
     * output is cleaned up, and bash/ksh and csh prompt setting is provided.
     *
     * @param prompt        the desired prompt - note that special characters
     *                      for any of the 3 shell variants should NOT be used
     *                      these include but are not limited to [$,@].
     * @exception IOException   if the connection to the server is unexpectedly
     *                          terminated
     */
    public void negotiatePrompt(String prompt) throws IOException
    {
        String response = null;
        final String TEST_STR = "This is a test";


        // Set the prompt to null to avoid blocking
        this.setPrompt(null);

        // Set prompt for bash/ksh
        send("PS1=\"" + prompt + "\"");
        // Seperate the two
        send("");
        // Set prompt for csh
        send("set prompt=\"" + prompt + "\"");

        stabilizePrompt(prompt);

        // Echo a test string to the server, to be able to get rid of
        // all unwanted output
        send("echo \"" + TEST_STR + "\"");
        response = waitfor(TEST_STR + "\r\n" + prompt);

        // Finally, set the prompt to allow proper command execution
        setPrompt(prompt);

    } // end: negotiatePrompt


    /**
     * Reads a response from the server and returns it to the client
     *
     * @return              the servers response
     * @exception IOException   if the connection to the server is unexpectedly
     *                          terminated
     */
    public String readResponse() throws IOException
    {
        byte[] response = new byte[1024];
        read(response);
        String str = new String(response);
        str = str.trim();
        return (str);

    } // end: readResponse


    /**
     * Sends a command to the server and returns a <code>TelnetResponse</code>
     * convenience object.
     *
     * @param cmd           the command to execute on the server
     * @exception IOException   if the connection to the server is unexpectedly
     *                          terminated
     */
    public TelnetResponse sendCommand(String cmd) throws IOException
    {
        return (new TelnetResponse(super.send(cmd)));

    } // end: sendCommand


    /**
     * Mutator for the system prompt that saves a copy of the prompt to allow
     * it to be retrieved (the superclass does not have an accessor).
     *
     * @param prompt        the new system prompt
     */
    public void setPrompt(String prompt)
    {
        super.setPrompt(prompt);
        _prompt = prompt;

    } // end: setPrompt


    /**
     * Writes empty strings to the telnet server until a stable prompt is
     * acheived.
     *
     * @param prompt        the prompt to stabilize to
     */
    public void stabilizePrompt(String prompt) throws IOException
    {
        String prevResponse = null;
        String response = null;

        // Keep sending an empty command until the prompt is found
        // This is necessary to ignore all system-specific messages
        // Note that a newline does not affect incrementing ([xx]) prompts
        while (prevResponse == null || (response.equals("") ||
                                        ! (response.equals(prevResponse) &&
                                           response.equals(prompt))))
        {
            send("");
            prevResponse = response;
            response = readResponse();
        }

    } // end: stabilizePrompt


} // end: TelnetClient
