/*
 * @(#)TelnetFileReceiver.java	1.0 2001/03/21
 *
 * Copyright (C) 2001 by Mark Cunningham, Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter.jta;


// Java imports
import java.io.*;
import sun.misc.*;

// Adapter imports
import net.sourceforge.journeyftp.adapter.*;


/**
 * Telnet implementation of asynchronous file receiver.
 *
 * @author  Mark Cunningham, Maru Newby
 * @version 1.0, 2001/03/21
 */
public class TelnetFileReceiver extends FileReceiver
{


    /**
     * Retrieves the identified filename from the current working directory of
     * the server and writes it to an output stream.
     *
     * If the offset is non-zero, the contents of that file will be written to
     * the output stream from that offset.  It is assumed that the output
     * stream has already been set to the desired offset.
     *
     * @exception TransferAdapterException  an error occured that terminated
     *                                      the file transfer
     * TODO - fix the number of bytes transferred
     * TODO - use temporary storage rather than in-memory streams
     */
    public void getFile() throws TransferAdapterException
    {
        final String UU_CMD = "uuencode";

        // Retrieve the parameters
        OutputStream out = this.getOutputStream();
        String fileName = this.getFileName();
        long offset = this.getOffset();
        VirtualFile file = getVirtualFile();

        // Retrieve the client object
        TelnetFileTransferConnection connection =
                            (TelnetFileTransferConnection)this.getConnection();
        TelnetClient client = connection.getTelnetClient();

        // Don't attempt to transfer directories
        if (file.isDirectory())
            throw (new IncorrectFileTypeException(
           getString(AbstractTransferAdapter.ERR_GET_FILE_INCORRECT_FILE_TYPE,
                      fileName)));

        // Don't attempt to transfer zero-length files
        if (file.getSize() == 0)
            return;

        // Throw an error if the offset it nonzero
        if (offset > 0 )
            throw (new ResumeNotSupportedException(
           getString(AbstractTransferAdapter.ERR_GET_FILE_RESUME_NOT_SUPPORTED,
                     fileName)));

        try
        {
            TelnetResponse response = null;

            // Throw an error if uuencode can not be invoked
            response = client.sendCommand(UU_CMD);
            if (response.getErrorLine().indexOf(
                           AbstractTransferAdapter.SH_COMMAND_NOT_FOUND) != -1)

                throw (new OperationFailedException(
               getString(AbstractTransferAdapter.ERR_GET_FILE_OPERATION_FAILED,
                          fileName), null, response.getMessage()));

            // Test if the file is readable
            response = client.sendCommand("more " + fileName + " > /dev/null");
            if (response.getErrorLine().indexOf(
                           AbstractTransferAdapter.SH_PERMISSION_DENIED) != -1)
                throw (new PermissionDeniedException(
              getString(AbstractTransferAdapter.ERR_GET_FILE_PERMISSION_DENIED,
                        fileName)));

            // Make a file retrieval request to the server
            String oldPrompt = client.getPrompt();

            // Remove the prompt to prevent reading of data by the send method
            client.setPrompt(null);

            // Send the uuencode command
            client.send(UU_CMD + " " + fileName + " " + fileName + ".uu");

            InputStream in = client.getInputStream();
            TelnetUUDecoder decoder = new TelnetUUDecoder();

            // Decode the message
            try
            {
                // Read in the first line and ignore it
                decoder._decodeLineSuffix(in, out);
                decoder._decodeBufferPrefix(in, out);
                decoder._decodeLineSuffix(in, out);
            }
            catch (CEFormatException ex)
            {
                client.abortTransfer(oldPrompt);
                throw (new PermissionDeniedException(
              getString(AbstractTransferAdapter.ERR_GET_FILE_PERMISSION_DENIED,
                        fileName)));
            }
            catch (IOException ioe)
            {
                client.abortTransfer(oldPrompt);
                operationFailed(AbstractTransferAdapter.OP_GET_FILE,
                                ioe,
                                "UUDecode: invalid uu prefix");
            }

            long transferred = 0;
            // Loop around decoding the lines that come back
            while (true)
            {
                if (transferAbortRequested())
                {
                    client.abortTransfer(oldPrompt);
                    throw (new TransferAbortedException(
                    getString(AbstractTransferAdapter.ERR_TRANSFER_ABORTED)));
                }
                try
                {
                    try
                    {
                        transferred += decoder._decodeBuffer(in, out);
                        if (transferred >= 1024)
                        {
                            notifyDataTransferred(transferred);
                            transferred = 0;
                        }

                    }
                    catch (CEFormatException ex)
                    {
                        client.abortTransfer(oldPrompt);
                        operationFailed(AbstractTransferAdapter.OP_GET_FILE,
                                        ex,
                                        "UUDecode: invalid uu atom");
                    }

                }
                catch (CEStreamExhausted ex)
                {
                    try
                    {
                        decoder._decodeLineSuffix(in, out);
                        break;
                    }
                    catch (CEFormatException cfe)
                    {
                        client.abortTransfer(oldPrompt);
                        operationFailed(AbstractTransferAdapter.OP_GET_FILE,
                                        cfe,
                                        "UUDecode: invalid buffer suffix");
                    }
                    catch (IOException ioe)
                    {
                        client.abortTransfer(oldPrompt);
                        operationFailed(AbstractTransferAdapter.OP_GET_FILE,
                                        ioe,
                                        "UUDecode: invali buffer suffix");
                    }
                }
            }

            if (transferred > 0)
                notifyDataTransferred(transferred);

            // Reset the old prompt and reset the terminal
            client.setPrompt(oldPrompt);
            client.send("");
            client.send("pwd");
            client.send("");

            // If no exception is thrown before here, transfer was successful
        }
        catch (IOException ioe)
        {
            operationFailed(AbstractTransferAdapter.OP_GET_FILE, ioe, "");
        }

    } // end: getFile


} // end: TelnetFileReceiver
