/*
 * @(#)WFSAdapter.java	1.0 2001/03/12
 *
 * Copyright (C) 2001 by David Tarc, Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter.sambawfs;


// Java imports
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.net.InetAddress;

// jcifs imports
import jcifs.smb.*;

// Adapter imports
import net.sourceforge.journeyftp.adapter.ConnectionFailedException;
import net.sourceforge.journeyftp.adapter.ConnectionInfo;
import net.sourceforge.journeyftp.adapter.ConnectionRefusedException;
import net.sourceforge.journeyftp.adapter.DirectoryExistsException;
import net.sourceforge.journeyftp.adapter.DirectoryNotEmptyException;
import net.sourceforge.journeyftp.adapter.FileExistsException;
import net.sourceforge.journeyftp.adapter.TransferSetupException;
import net.sourceforge.journeyftp.adapter.IncorrectFileTypeException;
import net.sourceforge.journeyftp.adapter.FileTransferObserver;
import net.sourceforge.journeyftp.adapter.HostNotFoundException;
import net.sourceforge.journeyftp.adapter.PermissionDeniedException;
import net.sourceforge.journeyftp.adapter.ResourceNotFoundException;
import net.sourceforge.journeyftp.adapter.TransferAdapter;
import net.sourceforge.journeyftp.adapter.TransferAdapterException;
import net.sourceforge.journeyftp.adapter.VirtualFile;
import net.sourceforge.journeyftp.adapter.*;


/**
 * samba jcifs (wfs) implementation of a journeyftp <code>TransferAdapter</code>.
 *
 * @author  David Tarc, Maru Newby
 * @version 1.0, 2001/03/15
 */
public class WFSAdapter extends AbstractTransferAdapter
{
    // Instance variables
    private SmbFile   _client = null;
    private Vector    _observers = new Vector();
    private String    _share = null;
    protected String    _url = null;

    /**
     * Default no-arg constructor.
     */
    public WFSAdapter()
    {
        super();
    }


    /**
     * Opens a connection to a server using the provided
     * <code>ConnectionInfo</code> object.  If no exception is thrown, a
     * connection was successfully made.
     *
     * @param info          the <code>ConnectionInfo</code> object
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception ConnectionRefusedException    the protocol was unable to
     *                                          negotiate a connection with the
     *                                          server
     * @exception HostNotFoundException         the server name could not be
     *                                          resolved
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception PermissionDeniedException     the protocol was unable to
     *                                          authenticate itself with the
     *                                          server
     */
    public void connect( ConnectionInfo info ) throws ConnectionFailedException,
                                                      ConnectionRefusedException,
                                                      HostNotFoundException,
                                                      OperationFailedException,
                                                      PermissionDeniedException
    {
        try {
            InetAddress.getByName( info.getHost() );
        } catch (UnknownHostException uhe)
        {
                throw new HostNotFoundException( getString( ERR_CONNECT_HOST_NOT_FOUND,
                                                            info.getHost() ) );
        }

        try {
            _share = info.getShareName();

            _url = "smb://" + info.getWorkgroup() + ";"
                            + info.getUsername() + ":"
                            + info.getPassword() + "@"
                            + info.getHost();

            // Try to connect
            _client = new SmbFile( _url + "/" + _share );

            if ( ! _client.exists() )
            {
                disconnect();
                throw new PermissionDeniedException( getString( ERR_CONNECT_PERMISSION_DENIED,
                                                                info.getHost() ) );
            }
        } catch ( UnknownHostException uhe )
        {
            disconnect();
            throw new ConnectionRefusedException( getString( ERR_CONNECT_CONNECTION_REFUSED,
                                                             info.getHost() ) );
        } catch ( IOException ioe )
        {
            disconnect();
            operationFailed(OP_CONNECT, ioe);
        }

    } // end: connect


    /**
     * Creates a directory in the current working directory of the server.
     *
     * @param dirName       the name of the directory to create
     *
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception DirectoryExistsException      a directory with the same name
     *                                          already exists
     * @exception FileExistsException           a file with the same name
     *                                          already exists
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception PermissionDeniedException     the specified directory could
     *                                          not be created with the
     *                                          provided credentials
     */
    public void createDirectory( String dirName )
                                               throws ConnectionFailedException,
                                                      PermissionDeniedException,
                                                      DirectoryExistsException,
                                                      FileExistsException,
                                                      OperationFailedException
    {
        try {
            if ( ! _client.canWrite() )
            {
                throw new PermissionDeniedException( getString( ERR_MKDIR_PERMISSION_DENIED,
                                                                dirName ) );
            }

            SmbFile newDir = new SmbFile( _client, dirName );
            if ( newDir.exists() )
            {
                if ( newDir.isDirectory() )
                {
                    throw new DirectoryExistsException( getString( ERR_MKDIR_DIR_EXISTS,
                                                              dirName ) );
                } else if ( newDir.isFile() )
                {
                    throw new FileExistsException( getString( ERR_MKDIR_FILE_EXISTS,
                                                              dirName ) );
                } else {
                    operationFailed( OP_MKDIR );
                }
            }
            if (! newDir.mkdir() )
            {
                operationFailed( OP_MKDIR );
            }
        } catch (IOException ioe) {
            operationFailed( OP_MKDIR, ioe );
        }

    } // end: createDirectory


    /**
     * Disconnects from the server.
     *
     * @exception ConnectionFailedException     reports any error that occured
     *                                          closing an active connection
     */
    public void disconnect()
    {
        _client = null;
        _url = null;
        _share = null;

    } // end: disconnect


    /**
     * Returns a prepared data connection for file transfers.
     *
     * @return              the data connection
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    protected FileTransferConnection getConnection()
                                              throws ConnectionFailedException,
                                                     OperationFailedException
    {
        return new WFSFileTransferConnection( _client );

    } // end: getConnection



    /**
     * Retrieves the identified filename from the current working directory of
     * the server and writes it to the given output stream.
     *
     * If the offset is non-zero, the contents of that file will be written to
     * the output stream from that offset.  It is assumed that the output stream
     * has already been set to the desired offset.
     *
     * Note that this method is designed to be asynchronous - using threads
     * and the <code>FileTransfer</code> object it should initiate a transfer
     * and communicate via the <code>TransferObserver</code> interface.
     *
     * @param fileName      the name of the file to retrieve
     * @param out           the output stream to write the file to
     * @param offset        the offset, in bytes, to read the file from
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated or did not exist
     */
/*    public void getFile( String fileName, OutputStream out, long offset )
                        throws ConnectionFailedException,
                               ResourceNotFoundException,
                               OperationFailedException
    {

        SmbFile fileToGet;
        try {
            fileToGet = new SmbFile(_client, fileName);
            if ( ! fileToGet.exists() )
                throw new ResourceNotFoundException();
            if ( ! fileToGet.isFile() )
                throw new ResourceNotFoundException();
        } catch (IOException ioe) {
            throw new ConnectionFailedException(ioe.getMessage());
        }
        WFSFileTransferConnection connection =
                                         new
                                         WFSFileTransferConnection(fileToGet);


        // Create and initialize the file receiver
        WFSFileReceiver receiver = new WFSFileReceiver();
        receiver.init(fileName, out, offset, connection);
        Enumeration enum = _observers.elements();
        while (enum.hasMoreElements())
            receiver.addObserver((FileTransferObserver)enum.nextElement());

        // Initialize and run the asynchronous transfer object
        Thread thread = new Thread(receiver);
        thread.run();

    } // end: getFile
*/


    /**
     * Returns an uninitialized <code>FileReceiver</code> object to transfer
     * a file from the server.
     *
     * @return              the file receiver
     */
    protected FileReceiver getFileReceiver()
    {
        return (new WFSFileReceiver());

    } // end: getFileReceiver


    /**
     * Returns an uninitialized <code>FileSender</code> object to transfer
     * a file to the server.
     *
     * @return              the file sender
     */
    protected FileSender getFileSender()
    {
        return (new WFSFileSender());

    } // end: getFileSender



    /**
     * Retrieves the path of the current working directory of the server.
     *
     * @return              the path of the server's current working directory
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     */
    public String getWorkingDirectory() throws ConnectionFailedException,
                                               OperationFailedException
    {
        return _client.getCanonicalPath().substring( _url.length() );
    } // end: getWorkingDirectory


    /**
     * Indicates whether the adapter is currently connected or not.
     *
     * @return              whether the adapter is currently connected
     */
    public boolean isConnected()
    {
        if ( _client != null )
        {
            try {
                SmbFile temp = new SmbFile( _client, "." );
                if ( temp.exists() )
                {
                    return true;
                } else
                {
                    disconnect();
                    return false;
                }
            } catch ( IOException ioe )
            {
                disconnect();
                return false;
            }
        } else
        {
            return false;
        }

    } // end: isConnected


    /**
     * Returns an array of <code>VirtualFile</code> objects, representing the
     * files and directories contained in the current working directory of the
     * server.
     *
     * @return              the array of <code>VirtualFile</code> objects
     * @exception ConnectionFailedException     an connection-related error
     *                                          prevented successful completion
     *                                          of the operation
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception ResourceNotFoundException     the working directory does not
     *                                          contain any files or the list
     *                                          cannot be parsed
     */
    public VirtualFile[] list() throws ConnectionFailedException,
                                       OperationFailedException,
                                       ResourceNotFoundException
    {
        String[] dirList = null;
        VirtualFile[] listOfVirtualFiles = null;

        dirList = _client.list();

        try
        {
            if ( dirList != null )
            {
                if ( dirList.length == 0) {
                    throw new ResourceNotFoundException ( getString( ERR_LIST_RESOURCE_NOT_FOUND ) );
                }
                listOfVirtualFiles = new VirtualFile[dirList.length];
                for ( int i = 0; i < dirList.length; i++ )
                {
                    SmbFile tempFile = new SmbFile( _client, dirList[i].trim() );
                    listOfVirtualFiles[i] = new VirtualFile();
                    listOfVirtualFiles[i].setName( tempFile.getName() );
                    listOfVirtualFiles[i].setIsDirectory( tempFile.isDirectory() );
                    if ( listOfVirtualFiles[i].isDirectory() )
                    {
                        listOfVirtualFiles[i].setSize( 0L );
                    } else
                    {
                        listOfVirtualFiles[i].setSize( tempFile.length() );
                    }
                    listOfVirtualFiles[i].setDateModified( new Date( tempFile.lastModified() ) );
                }
            } else
            {
                operationFailed( OP_LIST );
            }
        } catch ( IOException ioe ) {
            operationFailed( OP_LIST, ioe.getMessage() );
        }

        return listOfVirtualFiles;
    } // end: list

    /**
     * Returns an array of <code>VirtualFile</code> objects, representing the
     * the specified file if it is contained in the current working directory of the
     * server.
     *
     * @param fileName      the name of the file to list.
     * @return              the array of <code>VirtualFile</code> objects
     * @exception ConnectionFailedException     an connection-related error
     *                                          prevented successful completion
     *                                          of the operation
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception ResourceNotFoundException     the working directory does not
     *                                          contain this file
     */
    public VirtualFile listFile( String fileName )
                                   throws ConnectionFailedException,
                                          OperationFailedException,
                                          ResourceNotFoundException,
                                          IncorrectFileTypeException
    {
        VirtualFile fileToReturn = null;
        try
        {
            SmbFile tempFile = new SmbFile( _client, fileName );
            if ( ! tempFile.exists() )
            {
                throw new ResourceNotFoundException ( getString( ERR_LIST_RESOURCE_NOT_FOUND ) );
            }
            if ( ! tempFile.isDirectory() )
            {
                throw new IncorrectFileTypeException ( getString( ERR_LIST_FILE_INCORRECT_FILE_TYPE ) );
            }

            fileToReturn = new VirtualFile();
            fileToReturn.setName( tempFile.getName() );
            fileToReturn.setIsDirectory( false );
            fileToReturn.setSize( tempFile.length() );
            fileToReturn.setDateModified( new Date( tempFile.lastModified() ) );
        } catch ( IOException ioe ) {
            operationFailed( OP_LIST, ioe.getMessage() );
        }

        return fileToReturn;
    }

    /**
     * Sends the data from the provided input stream to the server, where it
     * will be written to a file identified by fileName in the current working
     * directory of the server.  If the file already exists on the server, it
     * will be overwritten if permitted.
     *
     * If the offset is non-zero, the contents of that file will be written to
     * the server from that offset.  It is assumed that the input stream
     * has already been set to the desired offset.
     *
     * Note that this method is designed to be asynchronous - using threads
     * and the <code>FileTransfer</code> object it should initiate a transfer
     * and communicate via the <code>TransferObserver</code> interface.
     *
     * @param fileName      the name of the file to be sent to the server
     * @param in            the input stream to read the file from
     * @param offset        the offset, in bytes, to write the file from
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated or did not exist
     */
/*
    public void putFile( String fileName, InputStream in, long offset )
                        throws ConnectionFailedException,
                               OperationFailedException
    {
        SmbFile fileToPut;
        try
        {
            fileToPut = new SmbFile( _client, fileName );
            if ( fileToPut.exists() )
                throw new ConnectionFailedException();
        } catch (IOException ioe) {
            throw new ConnectionFailedException();
        }

        WFSFileTransferConnection connection =
                                         new
                                         WFSFileTransferConnection(fileToPut);

        // Create and initialize the file receiver
        WFSFileSender sender = new WFSFileSender();
        sender.init(fileName, in, offset, connection);
        Enumeration enum = _observers.elements();
        while (enum.hasMoreElements())
            sender.addObserver((FileTransferObserver)enum.nextElement());

        // Initialize and run the asynchronous transfer object
        Thread thread = new Thread(sender);
        thread.run();

   } // end: putFile
*/


    /**
     * Removes the specified file/dir from the server.
     *
     * @param file          the <code>VirtualFile</code> object identifying
     *                      the remote file/dir to remove
     * @exception ConnectionFailedException     an connection-related error
     *                                          prevented successful completion
     *                                          of the operation
     * @exception DirectoryNotEmptyException    the specified directory was not
     *                                          empty
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception PermissionDeniedException     the specified file could
     *                                          not be removed with the
     *                                          provided credentials
     * @exception ResourceNotFoundException     the file targeted for
     *                                          removal could not be found
     */
    public void remove(VirtualFile file) throws ConnectionFailedException,
                                                DirectoryNotEmptyException,
                                                OperationFailedException,
                                                PermissionDeniedException,
                                                ResourceNotFoundException
    {
        String fileName = file.getName();
        try
        {
            SmbFile tempFile = new SmbFile( _client, fileName );
            if ( ! tempFile.exists() )
            {
                throw new ResourceNotFoundException ( getString( ERR_REMOVE_RESOURCE_NOT_FOUND,
                                                                 fileName ) );
            }
            if ( ! tempFile.canWrite() )
            {
                throw new PermissionDeniedException ( getString( ERR_REMOVE_PERMISSION_DENIED,
                                                                 fileName ) );
            }
            if ( tempFile.isDirectory() )
            {
                String[] dirList = tempFile.list();
                if ( dirList.length != 0)
                {
                    throw new DirectoryNotEmptyException ( getString( ERR_REMOVE_DIR_NOT_EMPTY,
                                                                      fileName ) );
                }
            }

            if ( ! tempFile.delete() )
            {
                operationFailed( OP_REMOVE );
            }
        } catch (IOException ioe)
        {
            operationFailed( OP_REMOVE, ioe.getMessage() );
        }
    }


    /**
     * Renames a file or directory
     *
     * @param     file       the file or directory to be removed
     * @param     newName    the name the file will be renamed to
     * @param     overwrite  if newName already exists, setting overwrite to
     *                       true will cause it to be deleted
     * @result    boolean    true if file was renamed, false otherwise
     * @exception ConnectionFailedException    the connection to the server
     *                                         was terminated or did not exist
     * @exception PermissionDeniedException    the user did not have permission
     *                                         to remove the file or directory
     * @exception ResourceNotFoundException    the file or directory
     *                                         does not exist
     */
    public void rename( String oldName, String newName )
                           throws ConnectionFailedException,
                                  PermissionDeniedException,
                                  ResourceNotFoundException,
                                  FileExistsException,
                                  DirectoryExistsException,
                                  OperationFailedException
    {
        try
        {
            SmbFile oldFile = new SmbFile( _client, oldName );

            if ( ! oldFile.exists() )
            {
                throw new ResourceNotFoundException( getString( ERR_RENAME_RESOURCE_NOT_FOUND,
                                                                oldName ) );
            }

            if ( ! oldFile.canWrite() )
            {
                throw new PermissionDeniedException( getString( ERR_RENAME_PERMISSION_DENIED ) );
            }

            SmbFile newFile = new SmbFile( _client, newName );

            if ( newFile.exists() )
            {
                if ( newFile.isDirectory() )
                {
                    throw new DirectoryExistsException( getString( ERR_RENAME_DIR_EXISTS,
                                                        oldName,
                                                        newName ) );
                } else if ( newFile.isFile() )
                {
                    throw new FileExistsException( getString( ERR_RENAME_FILE_EXISTS,
                                                              oldName,
                                                              newName ) );
                } else
                {
                    operationFailed ( OP_RENAME );
                }
            }

            if ( ! oldFile.renameTo( newFile ) )
            {
                operationFailed( OP_RENAME );
            }
        } catch (IOException ioe)
        {
            operationFailed( OP_RENAME, ioe.getMessage() );
        }
    }


    /**
     * Sets the current working directory on the server.
     *
     * @param path          the path of the new working directory
     * @exception ConnectionFailedException     the connection to the server
     *                                          was terminated
     * @exception IncorrectFileTypeException
     * @exception OperationFailedException      the operation was unable to
     *                                          complete successfully
     * @exception ResourceNotFoundException     the specified path does not
     *                                          exist or is unavailable on the
     *                                          server
     * @exception PermissionDeniedException     the specified path cannot be
     *                                          accessed with the provided
     *                                          credentials
     */
    public void setWorkingDirectory(String path)
                                    throws ConnectionFailedException,
                                           IncorrectFileTypeException,
                                           OperationFailedException,
                                           PermissionDeniedException,
                                           ResourceNotFoundException
    {
        try
        {
            SmbFile temp;
            if (path.startsWith( "/" ) )
            {
                temp = new SmbFile( _url + path );
            } else
            {
                temp = new SmbFile( _client, path );
            }

            if ( temp.exists() )
            {
                _client = temp;
            } else
            {
                throw new ResourceNotFoundException( path + ": directory does not exist" );
            }
        } catch (MalformedURLException ioe)
        {
//            throw new ConnectionFailedException( ioe.getMessage() );
        } catch (UnknownHostException ioe)
        {
            throw new ConnectionFailedException( ioe.getMessage() );
        } catch (IOException se)
        {
            throw new ConnectionFailedException( se.getMessage() );
        }
    } // end: setWorkingDirectory


} // end: WFSAdapter

