/*
 * @(#)WFSFileReceiver.java	1.0 2001/03/12
 *
 * Copyright (C) 2001 by David Tarc, Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter.sambawfs;


// Java imports
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

// jcifs imports
import jcifs.smb.*;

// Adapter imports
import net.sourceforge.journeyftp.adapter.ConnectionFailedException;
import net.sourceforge.journeyftp.adapter.FileReceiver;
import net.sourceforge.journeyftp.adapter.TransferAdapterException;
import net.sourceforge.journeyftp.adapter.TransferAbortedException;


/**
 * WFS implementation of asynchronous ftp file receiver.
 *
 * @author  David Tarc, Maru Newby
 * @version 1.0, 2001/03/15
 */
public class WFSFileReceiver extends FileReceiver
{
    // Class variables
    public static final int BUFF_SIZE = 8192;


    /**
     * Retrieves the identified filename from the current working directory of
     * the server and writes it to an output stream.
     *
     * If the offset is non-zero, the contents of that file will be written to
     * the output stream from that offset.  It is assumed that the output stream
     * has already been set to the desired offset.
     *
     * @exception TransferAdapterException  an error occured that terminated
     *                                      the file transfer
     */
    public void getFile() throws TransferAdapterException
    {
        byte[] buffer = new byte[BUFF_SIZE];
        int bytesRead;
        SmbFileInputStream smbIn;

        // Retrieve the parameters
        OutputStream out = this.getOutputStream();
        String fileName = this.getFileName();
        long offset = this.getOffset();

        // Retrieve the client object
        WFSFileTransferConnection connection =
                                (WFSFileTransferConnection)this.getConnection();
        SmbFile fileToGet = connection.getSmbFile( fileName );

        try
        {
            // Make a file retrieval request to the server
            smbIn = new SmbFileInputStream( fileToGet );


            // Skip to correct offset
            long bytesSkipped = smbIn.skip( offset );
            if ( bytesSkipped != offset )
            {
                throw new TransferAdapterException( "file could not be set to that offset" );
            }

            // Write out all the data, taking care to notify observers
            // of the bytes transferred.
            while ((! this.transferAbortRequested()) &&
                   ((bytesRead = smbIn.read(buffer, 0, buffer.length)) != -1))
            {
                out.write(buffer, 0, bytesRead);
                this.notifyDataTransferred(bytesRead);
            }

            // Won't work without this
            out.flush();
            out.close();

            // Check for an abort condition
            if (this.transferAbortRequested())
                throw new TransferAbortedException();

            // Close the stream and check for completion message
            smbIn.close();

            // If no exception is thrown before here, transfer was successful
        }
        catch (IOException ioe)
        {
            throw (new ConnectionFailedException());
        }

    } // end: getFile


} // end: WFSFileReceiver
